#!/usr/bin/env python
#
# This is the code for Grove - 12 Key Capacitive I2C Touch Sensor V2(MPR121).
# (https://www.seeedstudio.com/Grove-12-Key-Capacitive-I2C-Touch-Sensor-V2-MPR12-p-3141.html)
# which is is a multichannel proximity capacitive touch sensor.
# It's a 3-in-1 module with the following features:
#   Capacitance Sensing, Touch Sensing, and Proximity Sensing.
#
'''
## License
Author: Downey
The MIT License (MIT)

Grove 12 channel touch sensor MPR121 for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
import time
from grove.i2c import Bus


TOUCH_SENSOR_DEFAULT_ADDR                 = 0x5b

MODE_CONFIG_REG_ADDR                      = 0x5e
GLOBAL_PARAM_REG_ADDR_L                   = 0x5c
TOUCH_STATUS_REG_ADDR_L                   = 0x00
SET_DEBOUNCE_REG_ADDR                     = 0x5b

FILTERED_DATA_REG_START_ADDR_L            = 0x04
CHANNEL_NUM                               = 12

STOP_MODE                                 = 0
NORMAL_MODE                               = 0x3c

class Grove12KeyCapTouchMpr121():
    def __init__(self,bus_num = 1,addr = TOUCH_SENSOR_DEFAULT_ADDR):
        self.bus = Bus(bus_num)
        self.addr = addr
        self.threshold = 0
        self.touch_flag = [0]*CHANNEL_NUM

    def sensor_init(self):
        self._set_mode(STOP_MODE)
        data = [0x23,0x10]
        self._set_global_param(data)
        self._set_debounce(0x22)
        self._set_mode(NORMAL_MODE)

    def set_threshold(self,threshold):
        self.threshold = threshold

    def wait_for_ready(self):
        time.sleep(.2)

    def _set_mode(self,mode):
        self.bus.write_byte_data(self.addr,MODE_CONFIG_REG_ADDR,mode)
    
    def _set_global_param(self,data):
        self.bus.write_i2c_block_data(self.addr,GLOBAL_PARAM_REG_ADDR_L,data)
    
    def _set_debounce(self,data):
        self.bus.write_byte_data(self.addr,SET_DEBOUNCE_REG_ADDR,data)

    def _check_status_register(self):
        data_status = self.bus.read_i2c_block_data(self.addr,TOUCH_STATUS_REG_ADDR_L,2)
        return data_status
    
    def get_filtered_touch_data(self,sensor_status):
        result_value = []
        for i in range(CHANNEL_NUM):
            time.sleep(.01)
            if(sensor_status & (1<<i)):
                channel_data = self.bus.read_i2c_block_data(self.addr,FILTERED_DATA_REG_START_ADDR_L+2*i,2)
                result_value.append(channel_data[0] | channel_data[1]<<8 )
            else:
                result_value.append(0)
        return result_value

    def listen_sensor_status(self):
        data = self._check_status_register()
        touch_status = data[0] | (data[1]<<8) 
        touch_result_value = self.get_filtered_touch_data(touch_status)

        for i in range(CHANNEL_NUM):
            if(touch_result_value[i] < self.threshold ):
                touch_result_value[i] = 0
        return touch_result_value
    
    def parse_and_print_result(self,result):
        for i in range(CHANNEL_NUM):
            if(result[i] != 0):
                if(0 == self.touch_flag[i]):
                    self.touch_flag[i] = 1
                    print("Channel %d is pressed,value is %d" %(i,result[i]))
            else:
                if(1 == self.touch_flag[i]):
                    self.touch_flag[i] = 0
                    print("Channel %d is released,value is %d" %(i,result[i]))
        


mpr121 = Grove12KeyCapTouchMpr121() 
def main():
    mpr121.sensor_init()
    mpr121.set_threshold(0x60)
    mpr121.wait_for_ready()
    while 1:
        result = mpr121.listen_sensor_status()
        mpr121.parse_and_print_result(result)
        time.sleep(.1)

if __name__  == '__main__':
    main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
'''
This is the code for
    - `Grove - Water Sensor <https://www.seeedstudio.com/Grove-Water-Sensor-p-748.html>`_

Examples:

    .. code-block:: python

        import time
        from grove.grove_water_sensor import GroveWaterSensor

        # connect to alalog pin 2(slot A2)
        PIN = 2

        sensor = GroveWaterSensor(PIN)

        print('Detecting ...')
        while True:
        value = sensor.value
            if sensor.value > 800:
                print("{}, Detected Water.".format(value))
            else:
                print("{}, Dry.".format(value))

            time.sleep(.1)
'''
import time, sys, math
from grove.adc import ADC

__all__ = ["GroveWaterSensor"]

class GroveWaterSensor:
    '''
    Grove Water Sensor class

    Args:
        pin(int): number of analog pin/channel the sensor connected.
    '''
    def __init__(self, channel):
        self.channel = channel
        self.adc = ADC()

    @property
    def value(self):
        '''
        Get the water strength value

        Returns:
            (int): ratio, 0(0.0%) - 1000(100.0%)
        '''
        return self.adc.read(self.channel)

Grove = GroveWaterSensor


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.ADC)
    pin = sh.argv2pin()

    sensor = GroveWaterSensor(pin)

    print('Detecting ...')
    while True:
        value = sensor.value
        if sensor.value > 800:
            print("{}, Detected Water.".format(value))
        else:
            print("{}, Dry.".format(value))

        time.sleep(.1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
'''
This is the code for
    - `Grove - 5-Way Switch          <https://www.seeedstudio.com/Grove-5-Way-Switch-p-3136.html>`_
    - `Grove - 6-Position DIP Switch <https://www.seeedstudio.com/Grove-6-Position-DIP-Switch-p-3137.html>`_

Examples:
    .. code-block:: python

        from grove.button import Button
        import grove.grove_multi_switch.GroveMultiSwitch
        import time

        switch = GroveMultiSwitch()

        # the default behavior is
        # printing all happening event with names 
        # remove \'\'\' pairs below to begin your experiment
        \'\'\'
        # define a customized event handle your self
        def cust_on_event(index, event, tm):
            # switch.btn(ButtonTypedI2c object) could be used to check every event.
            print("event with code {}, time {}".format(event, tm))

        switch.on_event = cust_on_event
        \'\'\'
        while True:
            time.sleep(1)

'''
from __future__ import print_function
from grove.button import Button
from grove.factory import Factory
import time

__all__ = ['GroveMultiSwitch']

class GroveMultiSwitch(object):
    '''
    Grove 5-Way-Switch/6-Po-DIP-Switch class

    all of them has a button array with I2C interface.

    '''
    def __init__(self):
        self.btn = Factory.getButton("I2C", 0)
        self.__on_event = None
        self.btn.on_event(self, GroveMultiSwitch.__handle_event)

    @property
    def on_event(self):
        '''
        Argument
            callback -- a callable function/object,
                        will be called when there is button event
            callback prototype:
                callback(index, code, time)
            callback argument:
                Args:
                    index(int): button index, be in 0 to [button count - 1]

                    code (int): bits combination of
                              -  Button.EV_LEVEL_CHANGED
                              -  Button.EV_SINGLE_CLICK
                              -  Button.EV_DOUBLE_CLICK
                              -  Button.EV_LONG_PRESS

                    time(time): event generation time

                Returns: none

        Examples:
            set

            .. code-block:: python

                obj.on_event = callback

            get

            .. code-block:: python

                callobj = obj.on_event
        '''
        return self.__on_event

    @on_event.setter
    def on_event(self, callback):
        if not callable(callback):
            return
        self.__on_event = callback

    def __handle_event(self, evt):
        # print("event index:{} event:{} pressed:{}"
        #      .format(evt['index'], evt['code'], evt['pressed']))
        if callable(self.__on_event):
            self.__on_event(evt['index'], evt['code'], evt['time'])
            return

        name = self.btn.name(evt['index'])
        event = evt['code']
        print("{} : ".format(name), end='')
        print("HIGH " if evt['pressed'] else "LOW  ", end='')
        print("SINGLE-CLICK " if event & Button.EV_SINGLE_CLICK  else "", end='')
        print("DOUBLE-CLICK " if event & Button.EV_DOUBLE_CLICK  else "", end='')
        print("LONG - PRESS " if event & Button.EV_LONG_PRESS    else "", end='')
        print("LEVEL-CHANGE " if event & Button.EV_LEVEL_CHANGED else "", end='')
        print("")



def main():
    import time
    print(\
""" Make sure Grove-5-Way-Switch or Grove-6-Position-DIP-Switch
   inserted in one I2C slot of Grove-Base-Hat
""")

    switch = GroveMultiSwitch()

    # remove ''' pairs below to begin your experiment
    '''
    # define a customized event handle your self
    def cust_on_event(index, event, tm):
        # switch.btn could be used to check every event.
        print("event with code {}, time {}".format(event, tm))

    switch.on_event = cust_on_event
    '''
    while True:
        time.sleep(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# This is the code for Grove - Capacitive Touch Slide Sensor(CY8C4014LQI).
# (https://www.seeedstudio.com/Grove-Capacitive-Touch-Slider-Sensor-CY8C4014LQ-p-3183.html)
# which is a multifunctional touch sensor.
# You can touch the two buttons or slide on the 5-segment slider,
# each touch pad has a corresponding LED to indicate the touch status.
#
'''
## License
Author: Downey
The MIT License (MIT)

Grove touch sensor CY8C for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''

import time
from grove.i2c import Bus

TOUCH_SENSOR_CY8C_DEFAULT_IIC_ADDR        = 0X08
BUTTON_REG_ADDR                           = 0X00
TOUCH_SLIDER_REG_ADDR                     = 0X01

UNPRESSED                                 = 0
PRESSED                                   = 1

class GroveTouchSliderCy8c():
    
    left_button_press_flag = UNPRESSED
    right_button_press_flag = UNPRESSED

    def __init__(self,bus_num = 1, addr = TOUCH_SENSOR_CY8C_DEFAULT_IIC_ADDR):
        self.bus = Bus(bus_num)
        self.addr = addr
    
    def read_sensor_button_value(self):
        button_value = self.bus.read_byte_data(self.addr,BUTTON_REG_ADDR)
        return button_value

    def read_sensor_slider_value(self):
        slider_value = self.bus.read_byte_data(self.addr,TOUCH_SLIDER_REG_ADDR)
        return slider_value
    
    def parse_and_print_result(self,button_value,slider_value):

        '''
        if(0 == button_value):
            if(PRESSED == self.left_button_press_flag):
                print("Left button is released")
                self.left_button_press_flag = UNPRESSED
            if(PRESSED == self.right_button_press_flag):
                print("Right button is released")
                self.right_button_press_flag = UNPRESSED
        '''
        if(0x01 ==  button_value&0x1 ):
            if(UNPRESSED == self.left_button_press_flag): 
                print("Left button is pressed")
                self.left_button_press_flag = PRESSED
        elif(0 == button_value&0x1):
            if(PRESSED == self.left_button_press_flag):
                print("Left button is released")
                self.left_button_press_flag = UNPRESSED
        if(0x02 == button_value&0x2):
            if(UNPRESSED == self.right_button_press_flag): 
                print("Right button is pressed")
                self.right_button_press_flag = PRESSED
        elif(0 == button_value&0x2):
            if(PRESSED == self.right_button_press_flag):
                print("Right button is released")
                self.right_button_press_flag = UNPRESSED
        if(0 != slider_value):
            print("Slider is pressed,value is %s" %(slider_value))
    def listen_sensor_status(self):
        while True:
            button_value = self.read_sensor_button_value()
            slider_value = self.read_sensor_slider_value()
            self.parse_and_print_result(button_value,slider_value)
            time.sleep(.05)


CY8C = GroveTouchSliderCy8c()

def main():
     CY8C.listen_sensor_status()   


if __name__ == '__main__':
    main()
        



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
'''
This is the code for
    - `Grove -  Air Quality Sensor <https://www.seeedstudio.com/Grove-Air-quality-sensor-v1.3-p-2439.html)>`_

Examples:

    .. code-block:: python

        import time
        from grove.grove_air_quality_sensor_v1_3 import GroveAirQualitySensor

        # connect to alalog pin 2(slot A2)
        PIN = 2

        sensor = GroveAirQualitySensor(pin)

        print('Detecting ...') 
        while True:
            value = sensor.value        
            if value > 100:
                print("{}, High Pollution.".format(value))
            else:
                print("{}, Air Quality OK.".format(value))
            time.sleep(.1)
'''
import time, sys
from grove.adc import ADC

__all__ = ["GroveAirQualitySensor"]

class GroveAirQualitySensor(object):
    '''
    Grove Air Quality Sensor class

    Args:
        pin(int): number of analog pin/channel the sensor connected.
    '''
    def __init__(self, channel):
        self.channel = channel
        self.adc = ADC()

    @property
    def value(self):
        '''
        Get the air quality strength value, badest value is 100.0%.

        Returns:
            (int): ratio, 0(0.0%) - 1000(100.0%)
        '''
        return self.adc.read(self.channel)

Grove = GroveAirQualitySensor


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.ADC)
    pin = sh.argv2pin()

    sensor = GroveAirQualitySensor(pin)

    print('Detecting ...') 
    while True:
        value = sensor.value        
        if value > 100:
            print("{}, High Pollution.".format(value))
        else:
            print("{}, Air Quality OK.".format(value))
        time.sleep(.1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# This is the library for Grove Base Hat which used to connect grove sensors for raspberry pi.
#

'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
from grove.gpio import GPIO

class GPIOWrapper(GPIO):
    def __init__(self, pin, wrapper):
        self.__high_en = wrapper["high-enable"]
        self.__dir     = wrapper["direction"]
        self.__status_attr = ""
        self.__enable_attr = ""
        self.__disable_attr = ""
        if self.__dir == GPIO.IN:
            self.__status_attr  = wrapper["status-attr"]
        else:
            self.__enable_attr  = wrapper["enable-attr"]
            self.__disable_attr = wrapper["disable-attr"]
        super(GPIOWrapper, self).__init__(pin, self.__dir)

    def __is_enabled(self):
        v = self.read()
        return self.__high_en == bool(v)

    def is_enabled(self):
        return self.__is_enabled()

    def __enable(self):
        self.write(1 if self.__high_en else 0)

    def __disable(self):
        self.write(0 if self.__high_en else 1)

    def enable(self, _enable):
        if _enable:
            self.__enable()
        else:
            self.__disable()
        return

    def __getattr__(self, attr):
        if attr == self.__status_attr:
            return self.__is_enabled
        elif attr == self.__enable_attr:
            return self.__enable
        elif attr == self.__disable_attr:
            return self.__disable
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "enable":
            self.enable(value)
        else:
            self.__dict__[attr] = value

def main():
    import time
    from grove.helper import SlotHelper

    sh = SlotHelper(SlotHelper.GPIO)
    pin = sh.argv2pin()

    pir_motion_wrapper = {
        'high-enable' : True,
        'direction'   : GPIO.IN,
        'status-attr' : "has_motion",
        'enable-attr' : "nothing",
        'disable-attr': "nothing"
    }

    pir_motion = GPIOWrapper(pin, pir_motion_wrapper)
    while True:
        if pir_motion.has_motion():
            print("Hi, people is moving")
        else:
            print("Watching")
        time.sleep(1)

if __name__ == '__main__':
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
# Author: Zion Orent <zorent@ics.com>
# Copyright (c) 2015 Intel Corporation.
#
'''
This is the code for
    - Grove - Optical Rotary Encoder(TCUT1600X01) <https://www.seeedstudio.com/Grove-Optical-Rotary-Encoder-TCUT1600X0-p-3142.html>`_

Examples:

    .. code-block:: python

        from grove.grove_button import GroveButton
        import time, sys

        # connect to pin 5 (slot D5)
        PIN = 5
        encoder = GroveOpticalRotaryEncoder(PIN)

        # Read the value every second and detect motion
        while True:
            print("\rPosition: {0}  ".format(encoder.position()), file=sys.stderr, end='')
            time.sleep(0.001)

'''
from __future__ import print_function
import time, sys, signal, atexit
from grove.gpio import GPIO

__all__ = ["GroveOpticalRotaryEncoder"]

# The UPM version rotaryencoder has bug result in segment fault.
# This pure python version could work well.
class GroveOpticalRotaryEncoder(object):
    '''
    Grove optical Rotary Encoder(TCUT1600X01) class

    Args:
        pin(int): the number of gpio/slot your grove device connected.
    '''
    def __init__(self, pin1, pin2 = None):
        pin2 = pin1 + 1 if pin2 is None else pin2
        self.__gpio  = GPIO(pin1, GPIO.IN)
        self.__gpio2 = GPIO(pin2, GPIO.IN)
        self.__gpio.on_event = self.__gpio_event
        self._pos = 0

    # called by GPIO library
    def __gpio_event(self, pin, value):
        v1 = self.__gpio.read()
        if not v1: return
        v2 = self.__gpio2.read()
        self._pos += 1 if v2 else -1

    def position(self, pos = None):
        '''
        set or get the position counter

        Args:
            pos(int):
                optinal, the position counter to be set.

                if not specified, only get the current counter

        Returns:
            (int): current position counter
        '''
        if not pos is None:
            self._pos = pos
        return self._pos

def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.GPIO)
    pin = sh.argv2pin()

    '''
    from upm.pyupm_rotaryencoder import RotaryEncoder as GroveOpticalRotaryEncoder
    from mraa import getGpioLookup

    mraa_pin1 = getGpioLookup("GPIO%02d" % (pin + 0))
    mraa_pin2 = getGpioLookup("GPIO%02d" % (pin + 1))

    # Instantiate a Grove Rotary Encoder, using signal pins mraa_pin1 & mraa_pin2
    myRotaryEncoder = GroveOpticalRotaryEncoder(mraa_pin1, mraa_pin2);
    '''
    myRotaryEncoder = GroveOpticalRotaryEncoder(pin)

    ## Exit handlers ##
    # This function stops python from printing a stacktrace when you hit control-C
    def SIGINTHandler(signum, frame):
        raise SystemExit

    # This function lets you run code on exit, including functions from myRotaryEncoder
    def exitHandler():
        print("Exiting")
        sys.exit(0)

    # Register exit handlers
    atexit.register(exitHandler)
    signal.signal(signal.SIGINT, SIGINTHandler)

    # Read the value every second and detect motion
    counter = 0
    while True:
        print("\rPosition: {0}  ".format(myRotaryEncoder.position()), file=sys.stderr, end='')
        counter += 1
        if counter >= 5000:
            print("")
            counter = 0
        time.sleep(0.001)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# This is the library for Grove Base Hat.
#
# OS Scheduler Classes
#
'''
provide functions to promote process real-time priority or change back to default

## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
from __future__ import print_function
import sys

SCHED_OTHER  = 0
SCHED_FIFO   = 1
SCHED_RR     = 2
SCHED_BATCH  = 3
SCHED_IDLE   = 5


_os_import = True
max_param = None
if sys.version_info >= (3, 3):
    import os as osm

    max_priority = osm.sched_get_priority_max(SCHED_FIFO)
    max_param  = osm.sched_param(max_priority)
    norm_param = osm.sched_param(0)
else:
    try:
        import ctypes
        import ctypes.util
        osm = ctypes.cdll.LoadLibrary(ctypes.util.find_library('c'))
    except Exception:
        print("error: module %s unusable" % __name__);
        _os_import = False

if not max_param and _os_import:
    class _sched_param(ctypes.Structure):
        _fields_ = [ ('sched_priority', ctypes.c_int) ]
    max_param_c = _sched_param()
    max_priority = osm.sched_get_priority_max(SCHED_FIFO)
    # print("max priority = %d" % max_priority)
    max_param_c.sched_priority = max_priority
    max_param = ctypes.byref(max_param_c)

    norm_param_c = _sched_param()
    norm_param_c.sched_priority = 0
    norm_param = ctypes.byref(norm_param_c)


def set_max_priority():
    "Set current process to highest priority"
    if not _os_import:
        return False
    osm.sched_setscheduler(0, SCHED_FIFO, max_param)

def set_default_priority():
    "Set current process to default priority"
    if not _os_import:
        return False
    osm.sched_setscheduler(0, SCHED_OTHER, norm_param)


'''
class Sched(object):

    def __init__(self):
        "Initialize Sched object"
'''


if __name__ == '__main__':
    import time
    set_max_priority()
    time.sleep(1)
    set_default_priority()




"""
The sgp30 uses a dynamic baseline compensation algorithm and on-chip calibration parameters to provide two
complementary air quality signals. Based on the sensor signals a total VOC signal (TVOC) and a CO2 equivalent signal
(CO2eq) are calculated. Sending an “Init_air_quality” command starts the air quality measurement. After the
“Init_air_quality” command, a “Measure_air_quality” command has to be sent in regular intervals of 1s to ensure proper
operation of the dynamic baseline compensation algorithm.
"""
import time
from grove.modules.sgp30 import SGP30

# Create sensor instance
SGP30_1 = SGP30()

# Test sensor with measure_test command
print(hex(SGP30_1.measure_test()))

# Get feature set version
SGP30_1.get_feature_set_version()

# Get 48bit unique serial ID
SGP30_1.get_serial_id()

# Initialize sensor and set compensation baseline values from file
SGP30_1.init_air_quality()

# measure_air_quality command has to be sent regular intervals 1s to ensure proper operation of
# dynamic baseline correction algorithm
while 1:
    start_time = time.clock_gettime_ns(time.CLOCK_BOOTTIME)

    SGP30_1.measure_air_quality()
    print("\nCO2eq: " + str(SGP30_1.CO2eq) + " ppm\nTVOC: " + str(SGP30_1.TVOC) + " ppb")

    end_time = time.clock_gettime_ns(time.CLOCK_BOOTTIME)
    print("Measurement cycle length: " + str((end_time - start_time)/1000000) + "ms")

    # Get compensation baseline values and display them
    # Store them somewhere to
    SGP30_1.get_baseline()
    print("CO2eq baseline: " + str(SGP30_1.CO2eq_baseline) + ", TVOC baseline: " + str(SGP30_1.TVOC_baseline))


    end_time = time.clock_gettime_ns(time.CLOCK_BOOTTIME)
    print("Total cycle length: " + str((end_time - start_time)/1000000) + "ms")
    # Sleep so that next cycle will be after 1s
    time.sleep(1-((end_time - start_time)/1000000000))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
# This is the library for Grove Base Hat
# which used to connect grove sensors for Raspberry Pi.
'''
This is the code for
    - `Grove - 2.5A DC current sensor  <https://www.seeedstudio.com/Grove-2-5A-DC-Current-Sensor-ACS70331-p-2929.html>`_
    - `Grove - 5A AC/DC current sensor <https://www.seeedstudio.com/Grove-5A-DC-AC-Current-Sensor-ACS70331-p-2928.html>`_
    - `Grove - 10A current sensor      <https://www.seeedstudio.com/Grove-10A-DC-Current-Sensor-ACS725-p-2927.html>`_

Examples:
    .. code-block:: python

        import time
        from grove_current_sensor import Current

        pin = 0
        sensor_type = "2.5A"

        #if use 10A current sensor input: pin = 0 , sensor_type = "10A"
        if (sensor_type == "2.5A"):
            sensitivity = 1000.0 / 800.0
            Vref = 260
        if (sensor_type == "5A_DC"):
            sensitivity = 1000.0 / 200.0
            Vref = 1498
        if (sensor_type == "5A_AC"):
            sensitivity = 1000.0 / 200.0
            Vref = 1498
        if (sensor_type == "10A"):
            sensitivity = 1000.0 / 264.0
            Vref = 322
        averageValue = 500

        ADC = Current()
        while True:

            if(sensor_type == "5A_AC"):
                pin_voltage = ADC.get_nchan_vol_milli_data(pin,averageValue)
                current = ADC.get_nchan_AC_current_data(pin,sensitivity,Vref,averageValue)
            else:
                temp = ADC.get_nchan_current_data(pin,sensitivity,Vref,averageValue)
                current = temp[0]
                pin_voltage = temp[1]
        
            current = round(current)
            print("pin_voltage(mV):")
            print(pin_voltage)
            print("current(mA):")
            print(current)
            print()
            time.sleep(1)
    
'''

import sys
import time
from grove.i2c import Bus

ADC_DEFAULT_IIC_ADDR = 0X04

ADC_CHAN_NUM = 8

REG_RAW_DATA_START = 0X10
REG_VOL_START = 0X20
REG_RTO_START = 0X30

REG_SET_ADDR = 0XC0

__all__ = ['Current','Bus']

class Current():
    '''
    Grove Current Sensor class
    '''

    def __init__(self,bus_num=1,addr=ADC_DEFAULT_IIC_ADDR):
        '''
        Init iic.

        Args: 
            bus_num(int): the bus number;
            addr(int): iic address;
        '''
        self.bus = Bus(bus_num)
        self.addr = addr
  
    def get_nchan_vol_milli_data(self,n,averageValue):
        '''
        Get n chanel data with unit mV.

        :param int n: the adc pin.
        :param int averageValue: Average acquisition frequency.
        Returns: 
            int: voltage value
        '''
        val = 0
        for i in range(averageValue):
            data = self.bus.read_i2c_block_data(self.addr,REG_VOL_START+n,2)
            val += data[1]<<8|data[0]
        val = val / averageValue
        return val

    def get_nchan_current_data(self,n,sensitivity,Vref,averageValue):
        '''
        2.5A/5A DC/10A cunrrent sensor get n chanel data with unit mA.

        :param int n: the adc pin.
        :param float sensitivity: The coefficient by which voltage is converted into current.
        :param int Vref: Initial voltage at no load.
        :param int averageValue: Average acquisition frequency.
        Returns: 
            int: current value
        '''
        val = 0
        for i in range(averageValue):
            data = self.bus.read_i2c_block_data(self.addr,REG_VOL_START+n,2)
            val += data[1]<<8|data[0]
        val = val / averageValue
        currentVal = (val - Vref) * sensitivity
        return currentVal,val

    def get_nchan_AC_current_data(self,n,sensitivity,Vref,averageValue):
        '''
        5A current sensor AC output and get n chanel data with unit mA.

        :param int n: the adc pin.
        :param float sensitivity: The coefficient by which voltage is converted into current.
        :param int Vref: Initial voltage at no load.
        :param int averageValue: Average acquisition frequency.
        Returns: 
            int: current value
        '''
        sensorValue = 0
        for i in range(averageValue):
            data=self.bus.read_i2c_block_data(self.addr,REG_VOL_START+n,2)
            val=data[1]<<8|data[0]
            if(val > sensorValue):
                sensorValue=val
            time.sleep(0.00004)
        currentVal = ((sensorValue - Vref) * sensitivity)*0.707
        return currentVal   

ADC = Current()
def main():
    if(len(sys.argv) == 3):

        pin = int(sys.argv[1])
        sensor_type = sys.argv[2]
        if (pin < 8 and (sensor_type == "2.5A" or sensor_type == "5A_DC" or sensor_type == "5A_AC" or sensor_type == "10A") ):
            if (sensor_type == "2.5A"):
                sensitivity = 1000.0 / 800.0
                Vref = 260
            if (sensor_type == "5A_DC"):
                sensitivity = 1000.0 / 200.0
                Vref = 1498
            if (sensor_type == "5A_AC"):
                sensitivity = 1000.0 / 200.0
                Vref = 1498
            if (sensor_type == "10A"):
                sensitivity = 1000.0 / 264.0
                Vref = 322
            averageValue = 500

            while True:

                if(sensor_type == "5A_AC"):
                    pin_voltage = ADC.get_nchan_vol_milli_data(pin,averageValue)
                    current = ADC.get_nchan_AC_current_data(pin,sensitivity,Vref,averageValue)
                else:
                    temp = ADC.get_nchan_current_data(pin,sensitivity,Vref,averageValue)
                    current = temp[0]
                    pin_voltage = temp[1]

                current = round(current)
                print("pin_voltage(mV):")
                print(pin_voltage)
                print("current(mA):")
                print(current)
                print()
                time.sleep(1)
            
        else:
            print("parameter input error!")
            print("Please enter parameters for example: python grove_current_sensor 0 2.5A")
            print("parameter1: 0-7")
            print("parameter2: 2.5A/5A_DC/5A_AC/10A")
    
    else:
        print("Please enter parameters for example: python grove_current_sensor 0 2.5A")
        print("parameter1: 0-7")
        print("parameter2: 2.5A/5A_DC/5A_AC/10A")
    
    
if __name__ == '__main__':
    main()


#!/usr/bin/env python
#
# This is the library for Grove Base Hat.
#
# Temper(ature) Classes
#

'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
import math
import threading
from grove.i2c import Bus 
from grove.temperature import Temper

RES_LOW = 0x00
RES_MEDIUM = 0x01
RES_HIGH = 0x02
RES_PRECISION = 0x03

MCP9808_REG_AMBIENT_TEMP = 0x05

class TemperMCP9808(Temper):
    def __init__(self, address=0x18):
        self._addr = address
        self._bus = Bus()
        self._resolution = Temper.RES_1_2_CELSIUS

    def _derive_res(self, res):
        ares = -1
        if res >= Temper.RES_1_2_CELSIUS:
            ares = RES_LOW
        elif res >= Temper.RES_1_4_CELSIUS:
            ares = RES_MEDIUM
        elif res >= Temper.RES_1_8_CELSIUS:
            ares = RES_HIGH
        elif res >= Temper.RES_1_16_CELSIUS:
            ares = RES_PRECISION

        if ares < 0:
            return False
        self._bus.write_byte(self._addr, ares)
        # print("ares = {}".format(ares))
        return True

    @property
    def temperature(self):
        result = self._bus.read_word_data(self._addr, MCP9808_REG_AMBIENT_TEMP)
        # Swap the bytes
        data = (result & 0xff) << 8 | (result & 0xff00) >> 8
        # print("data = {}".format(data))
        # print("data = {}".format(hex(data)))
        # Check if the temperature is negative
        if data & 0x1000:
            data = -((data ^ 0x0FFF) + 1)
        else:
            data = data & 0x0fff
        return data / 16.0



#
# Library for Grove - PM2.5 PM10 detect sensor (HM3301)
#

'''
## License

The MIT License (MIT)

Copyright (C) 2018  Seeed Technology Co.,Ltd.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''




from smbus2 import SMBus , i2c_msg
from smbus2 import SMBusWrapper
import time


HM3301_DEFAULT_I2C_ADDR = 0x40
SELECT_I2C_ADDR = 0x88
DATA_CNT = 29

class Seeed_HM3301(object):
    def __init__(self,bus_nr = 1):
            
        self.PM_1_0_conctrt_std = 0         # PM1.0 Standard particulate matter concentration Unit:ug/m3
        self.PM_2_5_conctrt_std = 0         # PM2.5 Standard particulate matter concentration Unit:ug/m3
        self.PM_10_conctrt_std = 0          # PM10  Standard particulate matter concentration Unit:ug/m3
    
        self.PM_1_0_conctrt_atmosph = 0     #PM1.0 Atmospheric environment concentration ,unit:ug/m3
        self.PM_2_5_conctrt_atmosph = 0     #PM2.5 Atmospheric environment concentration ,unit:ug/m3
        self.PM_10_conctrt_atmosph = 0      #PM10  Atmospheric environment concentration ,unit:ug/m3


        with SMBusWrapper(bus_nr) as bus:
            write = i2c_msg.write(HM3301_DEFAULT_I2C_ADDR,[SELECT_I2C_ADDR])
            bus.i2c_rdwr(write)

    def read_data(self):        
        with SMBusWrapper(1) as bus:
            read = i2c_msg.read(HM3301_DEFAULT_I2C_ADDR,DATA_CNT)
            bus.i2c_rdwr(read)
            return list(read)

    def check_crc(self,data):
        sum = 0
        for i in range(DATA_CNT-1):
            sum += data[i]
        sum = sum & 0xff
        #print(sum)
        #print(data[28])
        return (sum==data[28])
    
    def parse_data(self,data):
        self.PM_1_0_conctrt_std = data[4]<<8 | data[5]
        self.PM_2_5_conctrt_std = data[6]<<8 | data[7]
        self.PM_10_conctrt_std = data[8]<<8 | data[9]
        
        self.PM_1_0_conctrt_atmosph = data[10]<<8 | data[11]          
        self.PM_2_5_conctrt_atmosph = data[12]<<8 | data[13]
        self.PM_10_conctrt_atmosph  = data[14]<<8 | data[15]

        print("PM1.0 Standard particulate matter concentration Unit:ug/m3 = %d" %self.PM_1_0_conctrt_std)
        print("PM2.5 Standard particulate matter concentration Unit:ug/m3 = %d" %self.PM_2_5_conctrt_std)
        print("PM10  Standard particulate matter concentration Unit:ug/m3 = %d" %self.PM_10_conctrt_std)
        
        print("PM1.0 Atmospheric environment concentration ,unit:ug/m3 = %d" %self.PM_1_0_conctrt_atmosph)
        print("PM2.5 Atmospheric environment concentration ,unit:ug/m3 = %d" %self.PM_2_5_conctrt_atmosph)
        print("PM10  Atmospheric environment concentration ,unit:ug/m3 = %d" %self.PM_10_conctrt_atmosph)
        print(" ")
        print(" ")
        print(" ")

'''
with SMBusWrapper(1) as bus:
    write=i2c_msg.write(0x40,[0x88])
    bus.i2c_rdwr(write)    
    
    while(1):
        read = i2c_msg.read(0x40,29)
        bus.i2c_rdwr(read)
        data = list(read)
        print type(data)
        print data
        time.sleep(1)
''' 
   
def main():
    print("################### NOTICE!!!! ############################")
    print("####### Please set the I2c speed to 20khz              ####")
    print("####### sudo vim /boot/config.txt                      ####")
    print("####### add content : dtparam=i2c_arm_baudrate=20000   ####")
    print("####### sudo reboot                                    ####")
    print("################### NOTICE!!!! ############################")
    print(" ")
    print(" ")
    print(" ")
    
    hm3301 = Seeed_HM3301()
    time.sleep(.1)
    while 1:
        data = hm3301.read_data()
        #print data
        if(hm3301.check_crc(data) != True):
            print("CRC error!") 
        hm3301.parse_data(data)
        time.sleep(3)  

if __name__ == '__main__':
    main()



 



#!/usr/bin/env python
#
# Library for i2c 8 channels hub TCA9548A
#

'''
## License

The MIT License (MIT)

Copyright (C) 2022  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''


from __future__ import print_function
import time
from grove.i2c import Bus

#          Channel STATUS
#  -------------------------------------------------------
# ┆  B7  ┆  B6  ┆  B5  ┆  B4  ┆  B3  ┆  B2  ┆  B1  ┆  B0  ┆                                                                
# ┆-------------------------------------------------------┆
# ┆  ch7 ┆  ch6 ┆ ch5  ┆  ch4 ┆  ch3 ┆  ch2 ┆  ch1 ┆  ch0 ┆ 
# ˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉˉ  




class TCA9548A(object):
    
    __TCA_CHANNEL_0=0x1
    __TCA_CHANNEL_1=0x2
    __TCA_CHANNEL_2=0x4
    __TCA_CHANNEL_3=0x8
    __TCA_CHANNEL_4=0x10
    __TCA_CHANNEL_5=0x20
    __TCA_CHANNEL_6=0x40
    __TCA_CHANNEL_7=0x80
    __TCA_CHANNEL_ALL=0xFF


    def __init__(self, address=0x70, i2c=None):
        self.address = address
        self.bus = Bus(i2c)

        print('ID: {}'.format(self.id))

        self._channels = 0
    
    def write(self, value):
        self.bus.write_i2c_block_data(self.address, value, [])
    
    def read(self):
        self._channels = self.bus.read_i2c_block_data(self.address, 1, 1)[0]

    def open_channel(self, channel):
        self._channels |= channel
        self.write(self._channels)
    
    def close_channel(self, channel):
        self._channels &= ~channel
        self.write(self._channels)
        
Grove = TCA9548A

def main():
    Grove.open_channel(Grove.__TCA_CHANNEL_0)
    i2c = Bus()
    
    for address in range(0, 127):
        try:
            rez = i2c.write_quick(address)
            print('0x{:02x} : {}'.format(address, rez))
        except IOError:
            pass

if __name__ == '__main__':
    main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
'''
This is the code for
    - `Grove - Light Sensor <https://www.seeedstudio.com/Grove-Light-Sensor-v1.2-p-2727.html>`_

Examples:

    .. code-block:: python

        import time
        from grove.grove_light_sensor import GroveLightSensor

        # connect to alalog pin 2(slot A2)
        PIN = 2

        sensor = GroveLightSensor(pin)

        print('Detecting light...')
        while True:
            print('Light value: {0}'.format(sensor.light))
            time.sleep(1)
'''
import time, sys, math
from grove.adc import ADC

__all__ = ["GroveLightSensor"]

class GroveLightSensor(object):
    '''
    Grove Light Sensor class

    Args:
        pin(int): number of analog pin/channel the sensor connected.
    '''
    def __init__(self, channel):
        self.channel = channel
        self.adc = ADC()

    @property
    def light(self):
        '''
        Get the light strength value, maximum value is 100.0%

        Returns:
            (int): ratio, 0(0.0%) - 1000(100.0%)
        '''
        value = self.adc.read(self.channel)
        return value

Grove = GroveLightSensor


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.ADC)
    pin = sh.argv2pin()

    sensor = GroveLightSensor(pin)

    print('Detecting light...')
    while True:
        print('Light value: {0}'.format(sensor.light))
        time.sleep(1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
'''
This is the code for
    - `Grove - Grove - Round Force Sensor FSR402 <https://www.seeedstudio.com/Grove-Round-Force-Sensor-FSR40-p-3110.html>`_

Examples:

    .. code-block:: python

        import time
        from grove.grove_round_force_sensor import GroveRoundForceSensor

        # connect to alalog pin 2(slot A2)
        PIN = 2

        sensor = GroveRoundForceSensor(PIN)

        while True:
            fsr = sensor.value
            print('FSR Value: {}'.format(fsr), end='')
            if fsr < 10:
                print(" - No pressure")
            elif fsr < 600:
                print(" - Light squeeze")
            else:
                print(" - Big squeeze")
            time.sleep(1.0)
'''
from __future__ import print_function
import math
import sys
import time
from grove.adc import ADC

__all__ = ["GroveRoundForceSensor"]

class GroveRoundForceSensor(ADC):
    '''
    Class for Grove - Round Force Sensor

    Args:
        pin(int): number of analog pin/channel the sensor connected.
    '''
    def __init__(self, channel):
        self.channel = channel
        self.adc = ADC()
    
    @property
    def value(self):
        '''
        Get the force strength value, the strengest value is 100.0%.

        Returns:
            (int): ratio, 0(0.0%) - 1000(100.0%)
        '''
        return self.adc.read(self.channel)


Grove = GroveRoundForceSensor


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.ADC)
    pin = sh.argv2pin()

    sensor = GroveRoundForceSensor(pin)

    while True:
        fsr = sensor.value
        print('FSR Value: {}'.format(fsr), end='')
        if fsr < 10:
            print(" - No pressure")
        elif fsr < 600:
            print(" - Light squeeze")
        else:
            print(" - Big squeeze")
        time.sleep(1.0)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# This is the library for Grove Base Hat.
#
# OneLed Classes
#

'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
import time
import threading
from grove.gpio import GPIO

class OneLed(object):
    MAX_BRIGHT = 100

    def __init__(self, pin):
        self._light = False
        self._bright = self.MAX_BRIGHT
        self.__blink_on = 0.0
        self.__blink_off = 0.0
        self.__thrd = None
        self.__thr_exit = False
        self._r, self._g, self._b = 255,255,255

    def __thrd_exit(self):
        if not self.__thrd is None:
            self.__thr_exit = True
            self.__thrd.join()
            self.__thrd = None

    def __del__(self):
        self.__thrd_exit()

    def __thrd_blink(self):
        while not self.__thr_exit:
            self._lighton(True)
            time.sleep(self.__blink_on)
            self._lighton(False)
            time.sleep(self.__blink_off)

    def blink(self, on = 0.5, off = 0.5):
        self.__blink_on = on
        self.__blink_off = off
        if on < 0.001 and off < 0.001:
            self.__thrd_exit()
            return

        if not self.__thrd is None:
            return

        self.__thr_exit = False
        self.__thrd = threading.Thread( \
                target = OneLed.__thrd_blink, \
                args = (self,))
        self.__thrd.setDaemon(True)
        self.__thrd.start()

    # To be derived
    def _lighton(self, b = True):
        pass

    def light(self, on = None):
        if type(on) != bool:
            return self._light
        self.__thrd_exit()
        self._light = on
        self._lighton(self._light)
        return self._light

    # To be derived
    def _setcolor(self, r, g, b):
        return False

    # Set color
    def color(self, r = None, g = 0, b = 0):
        if type(r) != int:
            return self._r, self._g, self._b
        if self._setcolor(r, g, b):
            self._r, self._g, self, _b = r, g, b
        return self._r, self._g, self._b

    @property
    def brightness(self):
        return self._bright

    # bright 0(turn off) - 100(max bright)
    @brightness.setter
    def brightness(self, bright):
        self._bright = bright


class OneLedTypedGpio(OneLed):
    def __init__(self, pin, high_enable = True):
        super(OneLedTypedGpio, self).__init__(pin)
        self.__high_en = high_enable
        self.__gpio = GPIO(pin, GPIO.OUT)

    def _lighton(self, b = True):
        v = self.__high_en == bool(b) and bool(self._bright)
        # print("write {} {}".format(self._bright, int(v)))
        self.__gpio.write(int(v))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2019  Seeed Technology Co.,Ltd.
#
# Author: Jon Trulson <jtrulson@ics.com>
# Copyright (c) 2016-2017 Intel Corporation.
'''
This code is for
    - `Grove - 3-Axis Digital Compass V2 <https://www.seeedstudio.com/s/Grove-3-Axis-Digital-Compass-V2-p-3034.html>`_

'''

from __future__ import print_function
import time
import bmm150
import math


def main():
    device = bmm150.BMM150()  # Bus number will default to 1
    while (1):
        x, y, z = device.read_mag_data()

        heading_rads = math.atan2(x, y)

        heading_degrees = math.degrees(heading_rads)

        print("Magnetometer x: {0:.2f}".format(x), end=' ')
        print(" y: {0:.2f}".format(y), end=' ')
        print(" z: {0:.2f}".format(z), end=' ')
        print(" uT")

        print('heading(axis_Y point to): {0:.2f} degree'.format(heading_degrees))
        time.sleep(.250)

if __name__ == '__main__':
    main()


"""
License
The MIT License (MIT)

Copyright (C) 2020 Ville Laine, Aikuiskoulutus Taitaja

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

from time import sleep, clock_gettime, CLOCK_REALTIME
from grove.i2c import Bus


class bme280:
    """
    Grove bme280 sensor control library.
    Objects:
        - set_mode(mode, t_sb)
        - set_oversampling(osrs_h, osrs_t, osrs_p)
        - set_filter(filter_coefficient)
        - set_spi(spi3w_en)
        - write_reset()
        - read_id()
        - read_status()
        - read_compensated_signals()
        - read_raw_signals()
        - set_pressure_calibration(level, pressure)
        - get_altitude(pressure)
    """

    # Constant Definitions
    # Modes
    MODE_SLEEP = 0b00
    MODE_FORCE = 0b01
    MODE_NORMAL = 0b11

    # Normal mode standby values
    t_sb_0_5 = 0b000
    t_sb_62_5 = 0b001
    t_sb_125 = 0b010
    t_sb_250 = 0b011
    t_sb_500 = 0b100
    t_sb_1000 = 0b101
    t_sb_10 = 0b110
    t_sb_20 = 0b111

    # IIR filter coefficient
    filter_0 = 0b000
    filter_2 = 0b001
    filter_4 = 0b010
    filter_8 = 0b011
    filter_16 = 0b100

    # SPI control
    SPI_ON = 0b01
    SPI_OFF = 0b00

    # Oversampling modes
    OVRS_x0 = 0b000
    OVRS_x1 = 0b001
    OVRS_x2 = 0b010
    OVRS_x4 = 0b011
    OVRS_x8 = 0b100
    OVRS_x16 = 0b101

    def __init__(self):
        # Rev 2 Pi, Pi 2 & Pi 3 uses bus 1,  Rev 1 Pi uses bus 0
        # bme280 default I2C Address is 0x76
        self.bus = Bus()
        self.address = 0x76
        self.mode = self.MODE_SLEEP
        self.t_sb = self.t_sb_1000
        self.last_meas = 0

        # Calibration data
        # Calibration data registers are 0x88-0xA1(25 bytes) and 0xE1-0xE7(7 bytes) total 32 bytes
        self.calib = [0x00]*32

        # Init raw data array and variables
        self.raw_data = [0x00]*8
        self.raw_temperature = 0x00000
        self.raw_pressure = 0x00000
        self.raw_humidity = 0x00000

        # Init parameters
        self.osrs_h = self.OVRS_x0
        self.osrs_t = self.OVRS_x0
        self.osrs_p = self.OVRS_x0

        # IIR filter and SPI interface are off by default
        self.filter = self.filter_0
        self.spi3w_en = self.SPI_OFF

        # Status word. Only has two significant bits bit0 = im_update and bit3 = measuring
        self.status = 0x00
        self.im_update = False
        self.measuring = False

        # Compensation values
        # Temperature trimming values
        self.dig_T1 = 0x0000
        self.dig_T2 = 0x0000
        self.dig_T3 = 0x0000
        # Pressure trimming params
        self.dig_P1 = 0x0000
        self.dig_P2 = 0x0000
        self.dig_P3 = 0x0000
        self.dig_P4 = 0x0000
        self.dig_P5 = 0x0000
        self.dig_P6 = 0x0000
        self.dig_P7 = 0x0000
        self.dig_P8 = 0x0000
        self.dig_P9 = 0x0000
        # Humidity trimming params
        self.dig_H1 = 0x00
        self.dig_H2 = 0x0000
        self.dig_H3 = 0x00
        self.dig_H4 = 0x0000
        self.dig_H5 = 0x0000
        self.dig_H6 = 0x00

        # Get the trimming values
        self.__read_calib()

        # Variables for compensated measurements
        self.temperature = 0x00000
        self.pressure = 0x00000
        self.humidity = 0x00000
        self.calibrated_temperature = 0x00000
        self.calibrated_pressure = 0x00000
        self.calibrated_humidity = 0x00000

        # Variables for measurement calibration
        self.calibration_temperature = 0
        self.calibration_pressure = 0
        self.calibration_humidity = 0

    #########################################
    # set bme280 operating mode             #
    #########################################
    def set_mode(self, mode=MODE_SLEEP, t_sb=t_sb_1000):
        # Writes ctrl_meas register with current temperature and pressure oversampling settings
        # Only changes the mode
        # If normal mode selected also sets the standby time in config register

        # Set class variables
        self.mode = mode
        self.t_sb = t_sb

        # If no measurements are enabled there is no point going into measurement
        if self.osrs_t + self.osrs_p + self.osrs_h == 0:
            print("No measurement enabled!\nSee set_oversampling()-function to enable measurement.")
            return 0

        try:
            # If normal mode set also t_sb(standby time)
            if self.mode == self.MODE_NORMAL:
                # Write normal mode standby time t_sb to config register
                self.__config(t_sb, self.filter, self.spi3w_en)
                # Write mode to ctr_meas register
                self.__ctrl_meas(self.osrs_t, self.osrs_p, self.mode)
            # Otherwise just change the mode in ctrl_meas register
            else:
                self.__ctrl_meas(self.osrs_t, self.osrs_p, self.mode)

            self.last_meas = clock_gettime(CLOCK_REALTIME)
            # Everything went well return 1
            return 1
        except Exception as e:
            # e = sys.exc_info()[0]
            print("<p>Error: %s</p>" % e)
            return 0

    #########################################################################
    # Set oversampling/enable measurement. OVRS_x0 disables the measurement #
    #########################################################################
    def set_oversampling(self, osrs_h=OVRS_x0, osrs_t=OVRS_x0, osrs_p=OVRS_x0):
        # Set oversampling variables
        self.osrs_h = osrs_h
        self.osrs_t = osrs_t
        self.osrs_p = osrs_p

        try:
            # Set humidity oversampling
            self.__ctrl_hum(self.osrs_h)
            # Set temperature and pressure oversampling
            self.__ctrl_meas(self.osrs_t, self.osrs_p, self.mode)
        except Exception as e:
            # e = sys.exc_info()[0]
            print("<p>Error: %s</p>" % e)
            return 0

        # Everything went well return 1
        return 1

    #########################################
    # Set internal IIR filter               #
    #########################################
    def set_filter(self, filter_coefficient=filter_0):
        self.filter = filter_coefficient

        try:
            # Write config register with new filter setting
            self.__config(self.t_sb, self.filter, self.spi3w_en)
            return 1
        except Exception as e:
            # print("Error in set_filter()")
            print("<p>Error: %s</p>" % e)
            return 0

    #########################################
    # Set internal SPI interface            #
    #########################################
    def set_spi(self, spi3w_en=SPI_OFF):
        self.spi3w_en = spi3w_en

        try:
            # Write config register with new spi setting
            self.__config(self.t_sb, self.filter, self.spi3w_en)
            return 1
        except Exception as e:
            # print("Error in set_filter()")
            print("<p>Error: %s</p>" % e)
            return 0

    #########################################
    # Reset command                         #
    #########################################
    def write_reset(self):
        register = 0xE0
        data = 0xB6    # Reset command to register, other values than 0xB6 has no effect
        r_len = 0
        delay = 10

        command = ([register, data], r_len, delay)

        try:
            self.read_write(command)
            return 1
        except Exception as e:
            # print("Error in set_filter()")
            print("<p>Error: %s</p>" % e)
            return 0

    #########################################
    # read ID register                      #
    #########################################
    def read_id(self):
        register = 0xD0
        data = 0x00
        r_len = 1
        delay = 10

        command = ([register, data], r_len, delay)

        try:
            return hex(self.read_write(command)[0])
        except Exception as e:
            # print("Error in set_filter()")
            print("<p>Error: %s</p>" % e)
            return 0

    #########################################
    # read STATUS register                  #
    #########################################
    def read_status(self):
        register = 0xF3
        data = 0x00
        r_len = 1
        delay = 10

        command = ([register, data], r_len, delay)

        try:
            self.status = self.read_write(command)[0]
            self.measuring = (self.status >> 3) & 0b1
            self.im_update = self.status & 0b1
            return [self.status, self.measuring, self.im_update]
        except Exception as e:
            # print("Error in set_filter()")
            print("<p>Error: %s</p>" % e)
            return 0

    #########################################
    # read calibration registers            #
    #########################################
    def __read_calib(self):
        # Calibration data registers are 0x88-0xA1(25 bytes) and 0xE1-0xE7(7 bytes) total 32 bytes
        # command format ([Register, Data Bytes], Number of bytes to read, Additional sleep(ms))
        # If Number of bytes to read > 0 then read_write function reads from I2C Bus
        # Otherwise Write operation is performed.
        register1 = 0x88
        register2 = 0xE1
        data = 0x00
        r_len1 = 26
        r_len2 = 7
        delay = 10

        command1 = ([register1, data], r_len1, delay)
        command2 = ([register2, data], r_len2, delay)

        try:
            self.calib[0:26] = self.read_write(command1)
            self.calib[26:32] = self.read_write(command2)
        except Exception as e:
            # print("Error in set_filter()")
            print("<p>Error: %s</p>" % e)
            return 0

        # Assign fetched data to trim parameters
        # Temperature trimming params
        # unsigned are ok, signed must be calculated with twos complement
        bytes_short = 16
        bytes_char = 8

        self.dig_T1 = (self.calib[1] << 8) + (self.calib[0] & 0xffff)
        self.dig_T2 = self.__twos_complement(((self.calib[3] << 8) + self.calib[2]), bytes_short)
        self.dig_T3 = self.__twos_complement((self.calib[5] << 8) + self.calib[4], bytes_short)

        # Pressure trimming params
        self.dig_P1 = ((self.calib[7] << 8)+self.calib[6]) & 0xffff
        self.dig_P2 = self.__twos_complement((self.calib[9] << 8) + self.calib[8], bytes_short)
        self.dig_P3 = self.__twos_complement((self.calib[11] << 8) + self.calib[10], bytes_short)
        self.dig_P4 = self.__twos_complement((self.calib[13] << 8) + self.calib[12], bytes_short)
        self.dig_P5 = self.__twos_complement((self.calib[15] << 8) + self.calib[14], bytes_short)
        self.dig_P6 = self.__twos_complement((self.calib[17] << 8) + self.calib[16], bytes_short)
        self.dig_P7 = self.__twos_complement((self.calib[19] << 8) + self.calib[18], bytes_short)
        self.dig_P8 = self.__twos_complement((self.calib[21] << 8) + self.calib[20], bytes_short)
        self.dig_P9 = self.__twos_complement((self.calib[23] << 8) + self.calib[22], bytes_short)

        # Humidity trimming params
        self.dig_H1 = self.calib[25] & 0xff
        self.dig_H2 = self.__twos_complement((self.calib[27] << 8) + self.calib[26], bytes_short)
        self.dig_H3 = self.calib[28] & 0xff
        self.dig_H4 = self.__twos_complement((self.calib[29] << 4) + (self.calib[30] & 0x0f), bytes_short)
        self.dig_H5 = self.__twos_complement((self.calib[31] << 4) + ((self.calib[30] & 0xf0) >> 4), bytes_short)
        self.dig_H6 = self.__twos_complement(self.calib[32], bytes_char)

        # Everything went well return 1
        return 1

    #########################################
    # read raw signals registers            #
    #########################################
    def read_raw_signals(self):
        # RAW Signals memory area is 0xF7-0xFE so there is 8 bytes
        # command format ([Register, Data Bytes], Number of bytes to read, Additional sleep(ms))
        # If Number of bytes to read > 0 then read_write function reads from I2C Bus
        # Otherwise Write operation is performed.
        register = 0xF7
        data = 0x00
        r_len = 8
        delay = 10

        command = ([register, data], r_len, delay)

        try:
            self.raw_data = self.read_write(command)
            self.raw_pressure = (((self.raw_data[0] << 8) + self.raw_data[1]) << 4) + ((self.raw_data[2] >> 4) & 0x0f)
            self.raw_temperature = ((((self.raw_data[3] << 8) + self.raw_data[4]) << 4)
                                    + ((self.raw_data[5] >> 4) & 0x0f))
            self.raw_humidity = ((self.raw_data[6] << 8) + self.raw_data[7])
        except Exception as e:
            # print("Error in set_filter()")
            print("<p>Error: %s</p>" % e)
            return 0

        # Everything went well return 1
        return 1

    #########################################
    # read compensated signals              #
    #########################################
    def read_compensated_signals(self):
        # Assuming that raw signals are already read
        # Compensation is performed like in datasheet appendix 8
        # 8.1 Compensation formulas in double precision floating point

        try:
            # Get Temperature t_fine value
            t_fine = self.__comp_temperature()
            self.temperature = t_fine / 5120.0  # Output value of “51.23” equals 51.23 DegC
            self.calibrated_temperature = (self.temperature
                                           + (0 if self.calibration_temperature is 0 else self.calibration_temperature))

            # Get pressure
            self.pressure = self.__comp_pressure(t_fine) / 100.0
            self.calibrated_pressure = (self.pressure
                                        + (0 if self.calibration_pressure is 0 else self.calibration_pressure))

            # Get Humidity
            self.humidity = self.__comp_humidity(t_fine)
            self.calibrated_humidity = (self.humidity
                                        + (0 if self.calibration_humidity is 0 else self.calibration_humidity))

            return 1
        except Exception as e:
            # print("Error in set_filter()")
            print("<p>Error: %s</p>" % e)
            return 0

    #########################################
    # Temperature compensation formulas     #
    # returns t_fine, because it is used in #
    # other compensation formulas           #
    #########################################
    def __comp_temperature(self):
        var1 = ((self.raw_temperature / 16384.0) - (self.dig_T1 / 1024.0)) * self.dig_T2
        var2 = ((((self.raw_temperature / 131072.0) - (self.dig_T1 / 8192.0))
                 * ((self.raw_temperature / 131072.0) - (self.dig_T1 / 8192.0))) * self.dig_T3)
        # t_fine = var1 + var2
        return var1 + var2

    #############################################################
    # Pressure compensation formulas                            #
    # Returns pressure in Pa as double.                         #
    # Output value of “96386.2” equals 96386.2 Pa = 963.862 hPa #
    #############################################################
    def __comp_pressure(self, t_fine):
        var1 = (t_fine / 2.0) - 64000.0
        var2 = var1 * var1 * self.dig_P6 / 32768.0
        var2 = var2 + var1 * self.dig_P5 * 2.0
        var2 = (var2 / 4.0) + (self.dig_P4 * 65536.0)
        var1 = (self.dig_P3 * var1 * var1 / 524288.0 + self.dig_P2 * var1) / 524288.0
        var1 = (1.0 + var1 / 32768.0) * self.dig_P1
        if var1 == 0.0:
            return 0    # avoid exception caused by division by zero

        p = 1048576.0 - self.raw_pressure
        p = (p - (var2 / 4096.0)) * 6250.0 / var1
        var1 = self.dig_P9 * p * p / 2147483648.0
        var2 = p * self.dig_P8 / 32768.0
        p = p + (var1 + var2 + self.dig_P7) / 16.0
        return p

    ##################################################
    # Humidity compensation formulas                 #
    # Returns humidity in %rH as as double.          #
    # Output value of “46.332” represents 46.332 %rH #
    ##################################################
    def __comp_humidity(self, t_fine):
        var_h = t_fine - 76800.0
        var_h = ((self.raw_humidity - (self.dig_H4 * 64.0 + (self.dig_H5 / 16384.0 * var_h)))
                 * (self.dig_H2 / 65536.0 * (1.0 + self.dig_H6 / 67108864.0 * var_h
                                             * (1.0 + self.dig_H3 / 67108864.0 * var_h))))
        var_h = var_h * (1.0 - self.dig_H1 * var_h / 524288.0)
        if var_h > 100.0:
            var_h = 100.0
        elif var_h < 0.0:
            var_h = 0.0

        return var_h

    #########################################
    # write ctrl_hum register               #
    #########################################
    def __ctrl_hum(self, osrs_h=OVRS_x0):
        register = 0xF2
        r_len = 0
        delay = 10

        # Make byte to send
        # data = ((0x00 << 3) + osrs_h)
        data = osrs_h

        command = ([register, data], r_len, delay)
        self.read_write(command)

    #########################################
    # write ctrl_meas register              #
    #########################################
    def __ctrl_meas(self, osrs_t=OVRS_x0, osrs_p=OVRS_x0, mode=MODE_SLEEP):
        register = 0xF4
        r_len = 0
        delay = 10

        # Make byte to send
        data = ((((osrs_t << 3) + osrs_p) << 2) + mode)

        command = ([register, data], r_len, delay)
        self.read_write(command)

    #########################################
    # write config register                 #
    #########################################
    def __config(self, t_sb, iir_filter, spi3w_en):
        register = 0xF5
        r_len = 0
        delay = 10

        # Make byte to send
        data = ((((t_sb << 3) + iir_filter) << 2) + spi3w_en)

        command = ([register, data], r_len, delay)
        self.read_write(command)

    ##############################################################
    # set_pressure_calibration                                   #
    # level = known level from sea                               #
    # pressure =  current local pressure normalized to sea level #
    ##############################################################
    def set_pressure_calibration(self, level, pressure):
        self.calibration_pressure = (pressure - level / 8) - self.pressure

    #########################################
    # get current altitude                  #
    # pressure = current sea level pressure #
    #########################################
    def get_altitude(self, pressure):
        return (pressure - self.calibrated_pressure) * 8

    ###########################################
    # read and write data to the bme280/(I2C) #
    ###########################################
    def read_write(self, command):
        if command[1] <= 0:
            # Write to I2C-bus
            # write_i2c_block_data(address, offset, [data_bytes])
            self.bus.write_i2c_block_data(self.address, command[0][0], command[0][1:])
            sleep(command[2]/1000)
            return 1
        else:
            # Read from I2C-Bus
            # read_i2c_block_data(address, register, number_of_data_bytes)
            return self.bus.read_i2c_block_data(self.address, command[0][0], command[1])

    #########################################
    # Twos complement calculation           #
    #########################################
    @staticmethod
    def __twos_complement(input_value, num_bits):
        """Calculates a two's complement integer from the given input value's bits."""

        mask = 2 ** (num_bits - 1)
        return -(input_value & mask) + (input_value & (mask-1))



"""
License
The MIT License (MIT)

Copyright (C) 2020 Ville Laine, Aikuiskoulutus Taitaja

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

from time import sleep
from grove.i2c import Bus
from .sgp30_functions import bytes_to_int, int_to_bytes, calc_crc8, validate_crc8


class SGP30:

    def __init__(self):
        self.bus = Bus()

        # SGB30 i2c default address is 0x58
        self.address = 0x58
        self.crc_status = [0, 0]
        self.CO2eq = 0
        self.TVOC = 0
        self.CO2eq_raw = 0
        self.TVOC_raw = 0
        self.CO2eq_baseline = 0
        self.TVOC_baseline = 0

    #########################################
    # Command functions                     #
    #########################################

    # Init air quality command
    # A new “Init_air_quality” command has to be sent after every power-up or soft reset.
    def init_air_quality(self):
        command = ([0x20, 0x03], 0, 10)
        try:
            self.read_write(command)
        except Exception:
            # On fail return 0
            print("sgp30 Init Fail!")
            return 0
        else:
            print("sgp30 Init Ok!")
            # Command pass return 1
            return 1

    # Measure compensated Co2eq and TVOC values
    # Has to be sent regular intervals 1s to ensure proper operation of dynamic baseline correction algorithm
    def measure_air_quality(self):
        command = ([0x20, 0x08], 6, 12)
        data = self.read_write(command)

        if self.crc_status[0] == 0:
            print("Measurement CRC check failed", self.crc_status)
            self.CO2eq = 0
            self.TVOC = 0
            return 0
        else:
            self.CO2eq = bytes_to_int(data[0:2])
            self.TVOC = bytes_to_int(data[3:5])
            return 1

    # Get compensation baseline values
    #
    def get_baseline(self):
        command = ([0x20, 0x15], 6, 10)
        try:
            data = self.read_write(command)
        except Exception:
            # On fail return 0
            print("get_baseline() => read_write() failed.")
            return 0

        if self.crc_status[0] == 0:
            print("Basevalue read CRC check failed", self.crc_status)
            return 0
        else:
            self.CO2eq_baseline = bytes_to_int(data[0:2])
            self.TVOC_baseline = bytes_to_int(data[3:5])
            return 1

    # Set compensation baseline values
    # Sets also variables sgp30(Instance).CO2eq_baseline and sgp30(Instance).TVOC_baseline
    def set_baseline(self, message):
        command = ([0x20, 0x1e], 0, 10)
        formatted_data = []

        # generate correct message format [HByte, LByte, CRC, HByte, LByte, CRC, ...]
        for number in message:
            tmp_bytes = int_to_bytes(number)
            crc = calc_crc8(tmp_bytes)
            formatted_data.extend(tmp_bytes)
            formatted_data.append(crc)

        # Extend command data with with message
        command[0].extend(formatted_data)

        try:
            self.read_write(command)
        except Exception:
            # On fail return 0
            print("set_baseline() => read_write() failed.")
            return 0
        else:
            self.TVOC_baseline = message[0]
            self.CO2eq_baseline = message[1]
            print("Compensation baseline values writen successfully.")
            return 1

    # Set humidity compensation value
    #
    def set_humidity(self, message):
        command = ([0x20, 0x61], 0, 10)
        formatted_data = []

        # generate correct message format [HByte, LByte, CRC, HByte, LByte, CRC, ...]
        for number in message:
            tmp_bytes = int_to_bytes(number)
            crc = calc_crc8(tmp_bytes)
            formatted_data.extend(tmp_bytes)
            formatted_data.append(crc)

        # Extend command data with with message
        command[0].extend(formatted_data)

        try:
            self.read_write(command)
        except Exception:
            # On fail return 0
            print("set_humidity() => read_write() failed.")
            return 0
        else:
            print("Humidity compensation value writen successfully.")
            return 1

    # Measure test
    # The command “Measure_test” which is included for integration and production line testing runs an on-chip
    # self-test. In case of a successful self-test the sensor returns the fixed data pattern 0xD400 (with correct CRC)
    def measure_test(self):
        command = ([0x20, 0x32], 3, 220)
        data = self.read_write(command)

        if self.crc_status[0] == 0:
            print("Measurement CRC check failed", self.crc_status)
            return 0
        else:
            print("On-chip self-test succesfull!")
            return bytes_to_int(data[0:2])

    def get_feature_set_version(self):
        command = ([0x20, 0x2f], 3, 2)
        data = self.read_write(command)

        print("Product type: ", '{0:08b}'.format(data[0])[0:4])
        print("Product version: ", '{0:08b}'.format(data[1]) + ", " + str(data[1]))

    def measure_raw_signals(self):
        command = ([0x20, 0x50], 6, 25)
        data = self.read_write(command)

        if self.crc_status[0] == 0:
            print("Measurement CRC check failed", self.crc_status)
            self.CO2eq_raw = 0
            self.TVOC_raw = 0
            return 0
        else:
            self.CO2eq_raw = bytes_to_int(data[0:2])
            self.TVOC_raw = bytes_to_int(data[3:5])
            return 1

    # Soft reset command
    # A sensor reset can be generated using the “General Call” mode according to I2C-bus specification.
    # It is important to understand that a reset generated in this way is not device specific.
    # All devices on the same I2C bus that support the General Call mode will perform a reset.
    def soft_reset(self):
        command = ([0x00, 0x06], 0, 10)
        self.read_write(command)

    # Get serial ID
    def get_serial_id(self):
        command = ([0x36, 0x82], 9, 10)
        data = self.read_write(command)

        if self.crc_status[0] == 0:
            print("Measurement CRC check failed", self.crc_status)
            return 0
        else:
            print("Get serial succesfull!")
            print("Serial part 1: ", '{0:016b}'.format(bytes_to_int(data[0:2])))
            print("Serial part 2: ", '{0:016b}'.format(bytes_to_int(data[3:5])))
            print("Serial part 3: ", '{0:016b}'.format(bytes_to_int(data[7:9])))
            return 1

    #########################################
    # read and write data to the sgp30      #
    #########################################
    def read_write(self, command):
        if command[1] <= 0:
            # Write to I2C-bus
            # write_i2c_block_data(address, offset, [data_bytes])
            # Note that second offset byte is [data_bytes][0]
            self.bus.write_i2c_block_data(self.address, command[0][0], command[0][1:])
            sleep(command[2]/1000)
            return 1
        else:
            # Read (Write and read) from I2C-Bus
            # write_i2c_block_data(address, offset, [data_bytes])
            # Note that second offset byte is [data_bytes][0]
            # read_i2c_block_data(address, offset, number_of_data_bytes)
            self.bus.write_i2c_block_data(self.address, command[0][0], [command[0][1]])
            sleep(command[2]/1000)
            data = self.bus.read_i2c_block_data(self.address, 0, command[1])
            self.crc_status = validate_crc8(data)
            return data



"""
License
The MIT License (MIT)

Copyright (C) 2020 Ville Laine, Aikuiskoulutus Taitaja

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

NOTE!!
README.md for setting the Raspberry Pi's serial line!!!
`/dev/ttyS0` (aka mini UART) is too slow!!!
"""
import threading
import time
from grove.modules.gps import GPS


#############################################################
# Thread that updates data from receiver to inner variables #
#############################################################
class UpdateGPSData(threading.Thread):
    def __init__(self, gps_ins, name="UpdateGPS", daemon=False):
        """ constructor, setting initial variables """
        self._stop_event = threading.Event()
        self._sleep_period = 0.05
        self.gps = gps_ins
        self.error = False
        self._fail_count = 0
        self._max_fail_count = 10
        self._max_fail_sleep = 3
        threading.Thread.__init__(self, name=name, daemon=daemon)

    def run(self):
        print("### Starting gps update Thread! ###")

        """ Main Control loop """
        while not self._stop_event.isSet():
            # Update gps data if there is data
            # Query from serial interface if gps receiver has sent data with new_data()
            if self.gps.new_data():
                thread_lock.acquire()
                if self.gps.read():
                    # If everything went well reset fail status
                    self._fail_count = 0
                thread_lock.release()
            else:
                # And wait for little while if data read failed before trying next time
                self._fail_count += 1
                time.sleep(min(self._sleep_period * self._fail_count, self._max_fail_sleep))

            if self._fail_count > self._max_fail_count:
                print("Data update failed last %d times!!\nSleeping %.3f" %
                      (self._fail_count, min(self._sleep_period * self._fail_count, self._max_fail_sleep)))
                self.error = True
            else:
                self.error = False

        print("### Exiting gps update Thread! ###")

    # Define custom join object to set the exit flag to the run object
    def join(self, timeout=None):
        """ Stop the thread """
        self._stop_event.set()
        threading.Thread.join(self, timeout)


#############################
# Thread that displays data #
#############################
class PrintGPSData(threading.Thread):
    def __init__(self, gps_ins, name="PrintGPSData", daemon=False):
        """ constructor, setting initial variables """
        self._stop_event = threading.Event()
        self._sleep_period = 1
        self.gps = gps_ins
        threading.Thread.__init__(self, name=name, daemon=daemon)

    def run(self):
        print("### Starting gps print Thread! ###")

        """ Main Control loop """
        while not self._stop_event.isSet():
            thread_lock.acquire()
            print("\n%s" % self.gps.utc_datetime)
            print("Satellites in view:", gps.GSV_satellites_in_view)
            print("Satellites used:", gps.GGA_satellites_used)
            print("Position:", self.gps.location_DDM)
            print("Speed: %s and Course: %s" % (gps.RMC_speed_over_ground, gps.RMC_course_over_ground))

            # Release the lock and then sleep
            thread_lock.release()
            time.sleep(self._sleep_period)

        print("### Exiting gps print Thread! ###")

    # Define custom join object to set the exit flag to the run object
    def join(self, timeout=None):
        """ Stop the thread """
        self._stop_event.set()
        threading.Thread.join(self, timeout)


# MAIN THREAD #
if __name__ == "__main__":
    # Connect to gps receiver
    gps = GPS('/dev/ttyAMA0', 9600, 1)

    # Create ThreadLock for synchronization
    # Acquiring the lock will block from other threads from processing before lock is released again
    # thread_lock.acquire()
    # thread_lock.release()
    thread_lock = threading.Lock()

    # Create instances of update and print threads
    # Threads can be made as daemonic threads =
    # When all non-daemonic threads are exited all daemonic threads are exited too
    # but we control the closing individually with join() and with the help of an thread event
    gps_update_thread = UpdateGPSData(gps, daemon=False)
    gps_print_thread = PrintGPSData(gps, daemon=False)

    try:
        # Start the threads
        gps_update_thread.start()
        gps_print_thread.start()

        # And wait for Keyboard Interrupt
        while True:
            if gps_update_thread.error:
                # Stop the print thread because there is no new data to be printed
                gps_print_thread.join()
            else:
                if not gps_print_thread.is_alive():
                    # Create and start print thread again
                    gps_print_thread = PrintGPSData(gps, daemon=False)
                    gps_print_thread.start()
            time.sleep(1)
    except KeyboardInterrupt:
        print("Keyboard interrupt")
    finally:
        # Stop the threads
        gps_print_thread.join()
        gps_update_thread.join()

    print("### Exiting Main Thread! ###")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd. 
'''
This code is for
    - `Grove - Mech Keycap <https://www.seeedstudio.com/Grove-Mech-Keycap-p-3138.html>`_

which is a mechanical switch with a build-in LED.
The 255 full color RGB LED makes it simple and easy to show the statues of your switch. 

Examples:
    .. code-block:: python

        from grove.button import Button
        import grove.grove_mech_keycap.GroveKeycap
        import time

        # slot/gpio number your device plugin
        pin = 12
        obj = GroveKeycap(pin)

        # the default behavior of led is
        #   single click - on
        #   double click - blink
        #   long press   - off
        # remove \'\'\' pairs below to begin your experiment
        \'\'\'
        # define a customized event handle your self
        def cust_on_event(index, event, tm):
            # obj.led could be used to operate led
            print("event with code {}, time {}".format(event, tm))

        obj.on_event = cust_on_event
        \'\'\'
        while True:
            time.sleep(1)
'''
import time
from grove.button import Button
from grove.factory import Factory

# sphinx autoapi required
__all__ = ["GroveKeycap"]

class GroveKeycap(object):
    '''
    Grove Mech Keycap class

    Args:
        pin(int): the number of gpio/slot your grove device connected.
                  for RPi, only 12 or 18 are acceptable.
    '''
    def __init__(self, pin):
        # High = pressed
        self.__btn = Factory.getButton("GPIO-HIGH", pin)
        # single WS2812 LED
        self.led = Factory.getOneLed("WS2812-PWM", pin + 1)
        self.__on_event = None
        self.__btn.on_event(self, GroveKeycap.__handle_event)

    @property
    def on_event(self):
        '''
        Property access with
            callback -- a callable function/object,
                        will be called when there is button event
            callback prototype:
                callback(index, code, time)
            callback argument:
                Args:
                    index(int): button index, be in 0 to [button count - 1]

                    code (int): bits combination of
                              -  Button.EV_LEVEL_CHANGED
                              -  Button.EV_SINGLE_CLICK
                              -  Button.EV_DOUBLE_CLICK
                              -  Button.EV_LONG_PRESS

                    time(time): event generation time

                Returns: none

        Examples:
            set

            .. code-block:: python

                obj.on_event = callback

            get

            .. code-block:: python

                callobj = obj.on_event
        '''
        return self.__on_event

    @on_event.setter
    def on_event(self, callback):
        if not callable(callback):
            return
        self.__on_event = callback

    def __handle_event(self, evt):
        # print("event index:{} event:{} pressed:{}"
        #      .format(evt['index'], evt['code'], evt['pressed']))
        if callable(self.__on_event):
            self.__on_event(evt['index'], evt['code'], evt['time'])
            return

        self.led.brightness = self.led.MAX_BRIGHT
        event = evt['code']
        if event & Button.EV_SINGLE_CLICK:
            self.led.light(True)
            print("turn on  LED")
        elif event & Button.EV_DOUBLE_CLICK:
            self.led.blink()
            print("blink    LED")
        elif event & Button.EV_LONG_PRESS:
            self.led.light(False)
            print("turn off LED")


Grove = GroveKeycap

def main():
    from grove import helper
    from grove.helper import helper
    helper.root_check()

    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.PWM)
    pin = sh.argv2pin()

    ledbtn = GroveKeycap(pin)

    # the default behavior of led is
    #   single click - on
    #   double click - blink
    #   long press   - off
    # remove ''' pairs below to begin your experiment
    '''
    # define a customized event handle your self
    def cust_on_event(index, event, tm):
        # obj.led could be used to operate led
        print("event with code {}, time {}".format(event, tm))

    ledbtn.on_event = cust_on_event
    '''
    while True:
        time.sleep(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# This is the code for Grove - I2C Thermocouple Amplifier (MCP9600).
# (https://www.seeedstudio.com/Grove-I2C-Thermocouple-Amplifier-MCP960-p-3199.html)
#
# which is a thermocouple-to-digital converter with integrated 
# cold-junction and I2C communication protocol. 
#
# @author Downey
#         Peter Yang <turmary@126.com>
#
# This is the library for Grove Base Hat which used to connect grove sensors for raspberry pi.
#
'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
from __future__ import division
from grove.i2c import Bus
import math
import socket
import ctypes
import time

REG_WIDTH                      = 0x2

VERSION_ID_REG_ADDR            = 0x20
THERM_SENS_CFG_REG_ADDR        = 0x05
DEVICE_CFG_REG_ADDR            = 0x06

HOT_JUNCTION_REG_ADDR          = 0x00
JUNCTION_TEMP_DELTA_REG_ADDR   = 0X01
COLD_JUNCTION_TEMP_REG_ADDR    = 0X02

STATUS_REG_ADDR                = 0x4
_STATUS_REG_BURST_COMPLETE     = 0x80
_STATUS_REG_TEMP_UPDATED       = 0x40
_STATUS_REG_INPUT_EXCEEDED     = 0x20

FILT_OFF                       = 0
FILT_MIN                       = 1
FILT_MID                       = 4
FILT_MAX                       = 7

COLD_JUNC_RESOLUTION_0_625     = 0<<7
COLD_JUNC_RESOLUTION_0_25      = 1<<7

ADC_18BIT_RESOLUTION           = 0<<5
ADC_16BIT_RESOLUTION           = 1<<5
ADC_14BIT_RESOLUTION           = 2<<5
ADC_12BIT_RESOLUTION           = 3<<5

BURST_1_SAMPLE                 = 0<<2
BURST_2_SAMPLE                 = 1<<2
BURST_4_SAMPLE                 = 2<<2
BURST_8_SAMPLE                 = 3<<2
BURST_16_SAMPLE                = 4<<2
BURST_32_SAMPLE                = 5<<2
BURST_64_SAMPLE                = 6<<2
BURST_128_SAMPLE               = 7<<2

NORMAL_OPERATION               = 0
SHUTDOWN_MODE                  = 1
BURST_MODE                     = 2


THER_TYPE_K                    = 0X0<<4
THER_TYPE_J                    = 0X1<<4
THER_TYPE_T                    = 0X2<<4
THER_TYPE_N                    = 0X3<<4
THER_TYPE_S                    = 0X4<<4
THER_TYPE_E                    = 0X5<<4
THER_TYPE_B                    = 0X6<<4
THER_TYPE_R                    = 0X7<<4

class GroveThermocoupleAmpMCP9600(object):
    def __init__(self, address = 0x60):
        self._addr = address
        self._bus  = Bus()
        self._therm = THER_TYPE_K
        self._junc = HOT_JUNCTION_REG_ADDR
        self._junc_res = COLD_JUNC_RESOLUTION_0_625

    def _read_version(self):
        version = self._bus.read_i2c_block_data(self._addr,VERSION_ID_REG_ADDR,2)
        return version

    def _set_filt_coefficients(self,coefficients):
        data = self._bus.read_byte_data(self._addr,THERM_SENS_CFG_REG_ADDR)
        data = (data & 0xF8) | coefficients
        self._bus.write_byte_data(self._addr,THERM_SENS_CFG_REG_ADDR,data)

    def _set_cold_junc_resolution(self,junc_res):
        data = self._bus.read_byte_data(self._addr,DEVICE_CFG_REG_ADDR)
        data = (data & 0x7F) | junc_res
        self._bus.write_byte_data(self._addr,DEVICE_CFG_REG_ADDR,data)
        self._junc_res = junc_res

    def _set_ADC_meas_resolution(self,res):
        data = self._bus.read_byte_data(self._addr,DEVICE_CFG_REG_ADDR)
        data = (data & 0x9F) | res
        self._bus.write_byte_data(self._addr,DEVICE_CFG_REG_ADDR,data)

    def _set_burst_mode_samp(self,samp):
        data = self._bus.read_byte_data(self._addr,DEVICE_CFG_REG_ADDR)
        data = (data & 0xE3) | samp
        self._bus.write_byte_data(self._addr,DEVICE_CFG_REG_ADDR,data)

    def _set_sensor_mode(self,mode):
        data = self._bus.read_byte_data(self._addr,DEVICE_CFG_REG_ADDR)
        data = (data & 0xFC) | mode
        self._bus.write_byte_data(self._addr,DEVICE_CFG_REG_ADDR,data)

    def get_config(self):
        config1 = self._bus.read_byte_data(self._addr,DEVICE_CFG_REG_ADDR)
        config2 = self._bus.read_byte_data(self._addr,THERM_SENS_CFG_REG_ADDR)
        return config1, config2

    def set_therm_type(self,therm_type):
        therm_cfg_data = self._bus.read_byte_data(self._addr,THERM_SENS_CFG_REG_ADDR)
        therm_cfg_data = (therm_cfg_data & 0x8F) | therm_type
        self._bus.write_byte_data(self._addr,THERM_SENS_CFG_REG_ADDR,therm_cfg_data)
        self._therm = therm_type

    def set_junc_type(self, junc):
        if not junc is None:
            self._junc = junc
        return self._junc

    def set_config(self,
        filter     = FILT_OFF,
        junc_res   = COLD_JUNC_RESOLUTION_0_625,
        adc_res    = ADC_14BIT_RESOLUTION,
        burst_smps = BURST_1_SAMPLE,
        oper_mode  = NORMAL_OPERATION
    ):
        self._set_filt_coefficients(filter)
        self._set_cold_junc_resolution(junc_res)
        self._set_ADC_meas_resolution(adc_res)
        self._set_burst_mode_samp(burst_smps)
        self._set_sensor_mode(oper_mode)
        return None

    def read(self):
        data = self._bus.read_word_data(self._addr, self._junc)
        # Big endian -> little endian
        data = socket.ntohs(data)
        # print("RAW = 0x%X" % data)

        # It's 16-bit 2's complement code
        temperature = ctypes.c_short(data).value / 16.0
        return temperature

Grove = GroveThermocoupleAmpMCP9600

def main():
    print(
""" Make sure Grove-I2C-Thermocouple-Amplifier-(MCP9600)
   inserted in one I2C slot of Grove-Base-Hat"""
)

    snr = GroveThermocoupleAmpMCP9600()

    print("""
To prevent I2C access error,
please append below line to /boot/config.txt
--------------------------------------------
dtparam=i2c_arm_baudrate=50000
--------------------------------------------
to set a 50Kbps I2C baudrate.
""" )
    snr.set_therm_type(THER_TYPE_K)
    snr.set_config()

    while 1:
        print("Temperature: {:.2f} C".format(snr.read()))
        time.sleep(1)

if __name__  == "__main__":
    main()



#!/usr/bin/env python
#
# This is the library for Grove Base Hat.
#
# OneLed Classes
#

'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
import time
import threading
from grove.gpio import GPIO
from grove.led.one_led import OneLed
from rpi_ws281x import *

class OneLedTypedWs2812(OneLed):
    def __init__(self, pin):
        ws2812_pins = { 12:0, 13:1, 18:0, 19:1}
        if not pin in ws2812_pins.keys():
            print("OneLedTypedWs2812: pin {} could not used with WS2812".format(pin))
            return
        super(OneLedTypedWs2812, self).__init__(pin)

        # LED strip configuration:
        LED_COUNT      = 1       # Number of LED pixels.
        LED_FREQ_HZ    = 800000  # LED signal frequency in hertz (usually 800khz)
        LED_DMA        = 10      # DMA channel to use for generating signal (try 10)
        LED_BRIGHTNESS = 255     # Set to 0 for darkest and 255 for brightest
        LED_INVERT     = False   # True to invert the signal (when using NPN transistor level shift)
        LED_CHANNEL    = ws2812_pins.get(pin) # set to '1' for GPIOs 13, 19, 41, 45 or 53

        # Create PixelStrip object with appropriate configuration.
        self.strip = PixelStrip(LED_COUNT, pin, LED_FREQ_HZ, LED_DMA, LED_INVERT, LED_BRIGHTNESS, LED_CHANNEL)
        # Intialize the library (must be called once before other functions).
        self.strip.begin()

    def _lighton(self, on):
        if not on:
            r,g,b = 0,0,0
        else:
            r = self._bright * self._r / OneLed.MAX_BRIGHT
            g = self._bright * self._g / OneLed.MAX_BRIGHT
            b = self._bright * self._b / OneLed.MAX_BRIGHT
        self.strip.setPixelColor(0, Color(r,g,b))
        self.strip.show()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
# This is the library for Grove Base Hat
# which used to connect grove sensors for Raspberry Pi.
'''
This is the code for
    - `OLED Display 1.12"    <https://www.seeedstudio.com/Grove-OLED-Display-1-1-p-824.html>`_
    - `OLED Display 1.12" V2 <https://www.seeedstudio.com/Grove-OLED-Display-1-12-V2-p-3031.html>`_

Examples:

    .. code-block:: python

        import time
        from grove.factory import Factory

        oled = Factory.getDisplay("SH1107G")
        rows, cols = oled.size()
        print("OLED model: {}".format(oled.name))
        print("OLED type : {} x {}".format(cols, rows))

        oled.setCursor(0, 0)
        oled.write("hello world!")
        oled.setCursor(0, cols - 1)
        oled.write('X')
        oled.setCursor(rows - 1, 0)
        for i in range(cols):
            oled.write(chr(ord('A') + i))

        time.sleep(3)
        oled.clear()
'''
from grove.display.base import *
# from upm.pyupm_lcd import *
from grove.i2c import Bus
import sys

# sphinx autoapi required
__all__ = ["SH1107G"]

BasicFont = [
        [0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00],
        [0x00,0x00,0x5F,0x00,0x00,0x00,0x00,0x00],
        [0x00,0x00,0x07,0x00,0x07,0x00,0x00,0x00],
        [0x00,0x14,0x7F,0x14,0x7F,0x14,0x00,0x00],
        [0x00,0x24,0x2A,0x7F,0x2A,0x12,0x00,0x00],
        [0x00,0x23,0x13,0x08,0x64,0x62,0x00,0x00],
        [0x00,0x36,0x49,0x55,0x22,0x50,0x00,0x00],
        [0x00,0x00,0x05,0x03,0x00,0x00,0x00,0x00],
        [0x00,0x1C,0x22,0x41,0x00,0x00,0x00,0x00],
        [0x00,0x41,0x22,0x1C,0x00,0x00,0x00,0x00],
        [0x00,0x08,0x2A,0x1C,0x2A,0x08,0x00,0x00],
        [0x00,0x08,0x08,0x3E,0x08,0x08,0x00,0x00],
        [0x00,0xA0,0x60,0x00,0x00,0x00,0x00,0x00],
        [0x00,0x08,0x08,0x08,0x08,0x08,0x00,0x00],
        [0x00,0x60,0x60,0x00,0x00,0x00,0x00,0x00],
        [0x00,0x20,0x10,0x08,0x04,0x02,0x00,0x00],
        [0x00,0x3E,0x51,0x49,0x45,0x3E,0x00,0x00],
        [0x00,0x00,0x42,0x7F,0x40,0x00,0x00,0x00],
        [0x00,0x62,0x51,0x49,0x49,0x46,0x00,0x00],
        [0x00,0x22,0x41,0x49,0x49,0x36,0x00,0x00],
        [0x00,0x18,0x14,0x12,0x7F,0x10,0x00,0x00],
        [0x00,0x27,0x45,0x45,0x45,0x39,0x00,0x00],
        [0x00,0x3C,0x4A,0x49,0x49,0x30,0x00,0x00],
        [0x00,0x01,0x71,0x09,0x05,0x03,0x00,0x00],
        [0x00,0x36,0x49,0x49,0x49,0x36,0x00,0x00],
        [0x00,0x06,0x49,0x49,0x29,0x1E,0x00,0x00],
        [0x00,0x00,0x36,0x36,0x00,0x00,0x00,0x00],
        [0x00,0x00,0xAC,0x6C,0x00,0x00,0x00,0x00],
        [0x00,0x08,0x14,0x22,0x41,0x00,0x00,0x00],
        [0x00,0x14,0x14,0x14,0x14,0x14,0x00,0x00],
        [0x00,0x41,0x22,0x14,0x08,0x00,0x00,0x00],
        [0x00,0x02,0x01,0x51,0x09,0x06,0x00,0x00],
        [0x00,0x32,0x49,0x79,0x41,0x3E,0x00,0x00],
        [0x00,0x7E,0x09,0x09,0x09,0x7E,0x00,0x00],
        [0x00,0x7F,0x49,0x49,0x49,0x36,0x00,0x00],
        [0x00,0x3E,0x41,0x41,0x41,0x22,0x00,0x00],
        [0x00,0x7F,0x41,0x41,0x22,0x1C,0x00,0x00],
        [0x00,0x7F,0x49,0x49,0x49,0x41,0x00,0x00],
        [0x00,0x7F,0x09,0x09,0x09,0x01,0x00,0x00],
        [0x00,0x3E,0x41,0x41,0x51,0x72,0x00,0x00],
        [0x00,0x7F,0x08,0x08,0x08,0x7F,0x00,0x00],
        [0x00,0x41,0x7F,0x41,0x00,0x00,0x00,0x00],
        [0x00,0x20,0x40,0x41,0x3F,0x01,0x00,0x00],
        [0x00,0x7F,0x08,0x14,0x22,0x41,0x00,0x00],
        [0x00,0x7F,0x40,0x40,0x40,0x40,0x00,0x00],
        [0x00,0x7F,0x02,0x0C,0x02,0x7F,0x00,0x00],
        [0x00,0x7F,0x04,0x08,0x10,0x7F,0x00,0x00],
        [0x00,0x3E,0x41,0x41,0x41,0x3E,0x00,0x00],
        [0x00,0x7F,0x09,0x09,0x09,0x06,0x00,0x00],
        [0x00,0x3E,0x41,0x51,0x21,0x5E,0x00,0x00],
        [0x00,0x7F,0x09,0x19,0x29,0x46,0x00,0x00],
        [0x00,0x26,0x49,0x49,0x49,0x32,0x00,0x00],
        [0x00,0x01,0x01,0x7F,0x01,0x01,0x00,0x00],
        [0x00,0x3F,0x40,0x40,0x40,0x3F,0x00,0x00],
        [0x00,0x1F,0x20,0x40,0x20,0x1F,0x00,0x00],
        [0x00,0x3F,0x40,0x38,0x40,0x3F,0x00,0x00],
        [0x00,0x63,0x14,0x08,0x14,0x63,0x00,0x00],
        [0x00,0x03,0x04,0x78,0x04,0x03,0x00,0x00],
        [0x00,0x61,0x51,0x49,0x45,0x43,0x00,0x00],
        [0x00,0x7F,0x41,0x41,0x00,0x00,0x00,0x00],
        [0x00,0x02,0x04,0x08,0x10,0x20,0x00,0x00],
        [0x00,0x41,0x41,0x7F,0x00,0x00,0x00,0x00],
        [0x00,0x04,0x02,0x01,0x02,0x04,0x00,0x00],
        [0x00,0x80,0x80,0x80,0x80,0x80,0x00,0x00],
        [0x00,0x01,0x02,0x04,0x00,0x00,0x00,0x00],
        [0x00,0x20,0x54,0x54,0x54,0x78,0x00,0x00],
        [0x00,0x7F,0x48,0x44,0x44,0x38,0x00,0x00],
        [0x00,0x38,0x44,0x44,0x28,0x00,0x00,0x00],
        [0x00,0x38,0x44,0x44,0x48,0x7F,0x00,0x00],
        [0x00,0x38,0x54,0x54,0x54,0x18,0x00,0x00],
        [0x00,0x08,0x7E,0x09,0x02,0x00,0x00,0x00],
        [0x00,0x18,0xA4,0xA4,0xA4,0x7C,0x00,0x00],
        [0x00,0x7F,0x08,0x04,0x04,0x78,0x00,0x00],
        [0x00,0x00,0x7D,0x00,0x00,0x00,0x00,0x00],
        [0x00,0x80,0x84,0x7D,0x00,0x00,0x00,0x00],
        [0x00,0x7F,0x10,0x28,0x44,0x00,0x00,0x00],
        [0x00,0x41,0x7F,0x40,0x00,0x00,0x00,0x00],
        [0x00,0x7C,0x04,0x18,0x04,0x78,0x00,0x00],
        [0x00,0x7C,0x08,0x04,0x7C,0x00,0x00,0x00],
        [0x00,0x38,0x44,0x44,0x38,0x00,0x00,0x00],
        [0x00,0xFC,0x24,0x24,0x18,0x00,0x00,0x00],
        [0x00,0x18,0x24,0x24,0xFC,0x00,0x00,0x00],
        [0x00,0x00,0x7C,0x08,0x04,0x00,0x00,0x00],
        [0x00,0x48,0x54,0x54,0x24,0x00,0x00,0x00],
        [0x00,0x04,0x7F,0x44,0x00,0x00,0x00,0x00],
        [0x00,0x3C,0x40,0x40,0x7C,0x00,0x00,0x00],
        [0x00,0x1C,0x20,0x40,0x20,0x1C,0x00,0x00],
        [0x00,0x3C,0x40,0x30,0x40,0x3C,0x00,0x00],
        [0x00,0x44,0x28,0x10,0x28,0x44,0x00,0x00],
        [0x00,0x1C,0xA0,0xA0,0x7C,0x00,0x00,0x00],
        [0x00,0x44,0x64,0x54,0x4C,0x44,0x00,0x00],
        [0x00,0x08,0x36,0x41,0x00,0x00,0x00,0x00],
        [0x00,0x00,0x7F,0x00,0x00,0x00,0x00,0x00],
        [0x00,0x41,0x36,0x08,0x00,0x00,0x00,0x00],
        [0x00,0x02,0x01,0x01,0x02,0x01,0x00,0x00],
        [0x00,0x02,0x05,0x05,0x02,0x00,0x00,0x00],
]

class SH1107G(Display):
    '''
    OLED Display 1.12"(v2) use chip SH1107G.

    Args:
        address(int): I2C device address, default to 0x3E.
    '''
    MAX_GRAY    = 100
    _REG_CMD    = 0x00
    _REG_DATA   = 0x40
    _PAGE_CNT   = 16
    _PAGE_BYTES = 128
    _TOTAL_BYTES= _PAGE_CNT * _PAGE_BYTES
    
    _DISPLAY_CMD_OFF = 0xAE;
    _DISPLAY_CMD_ON = 0xAF;

    _BASE_LOW_COLUMN_ADDR = 0x00;
    _BASE_HIGH_COLUMN_ADDR = 0x10;
    _BASE_PAGE_START_ADDR = 0xB0;
    def __init__(self, address = 0x3C):
        super(SH1107G, self).__init__()
        self._bus = Bus()
        self._addr = address

        if self._bus.write_byte(self._addr, 0):
            print("Check if the OLED SH1107G inserted, then try again")
            sys.exit(1)
 
        id = self._bus.read_byte_data(self._addr, SH1107G._REG_CMD)
        # print(" id = 0x{:2x}".format(id))
        self._sh1107 = (id & 0x3F) == 0x07

        blk =     [0xAE]   # Display OFF
        blk.append(0xD5)   # Set Dclk
        blk.append(0x50)   # 100Hz
        blk.append(0x20)   # Set row address
        blk.append(0x81)   # Set contrast control
        blk.append(0x80)
        blk.append(0xA0)   # Segment remap
        blk.append(0xA4)   # Set Entire Display ON 
        blk.append(0xA6)   # Normal display
        blk.append(0xAD)   # Set external VCC
        blk.append(0x80)
        blk.append(0xC0)   # Set Common scan direction
        blk.append(0xD9)   # Set phase leghth
        blk.append(0x1F)
        blk.append(0xDB)   # Set Vcomh voltage
        blk.append(0x27)
        blk.append(0xAF)   # Display ON
        blk.append(0xB0)
        blk.append(0x00)
        blk.append(0x10)
        self._cmds(blk)
        self.clear()

    def _cmd(self, cmd):
        try:
            self._bus.write_byte_data(self._addr,
                                    SH1107G._REG_CMD, cmd)
        except IOError:
            print("*** Check if OLED module inserted ***")
            sys.exit(1)

    def _cmds(self, cmds):
        for c in cmds:
            self._cmd(c)

    def _datas(self, datas):
        length = len(datas)
        data = bytearray(length + 1)
        data[0] = SH1107G._REG_DATA
        for i in range(length):
            data[i + 1] = datas[i]
        try:
            self._bus.write_i2c_block_data(self._addr,
                                  SH1107G._REG_DATA, datas)
        except IOError:
            print("*** Check if OLED module inserted ***")
            sys.exit(1)

    @property
    def name(self):
        '''
        Get device name

        Returns:
            string: SH1107G/SSD1307 depends your device plugin.
        '''
        return "SH1107G"

    def type(self):
        '''
        Get device type

        Returns:
            int: ``TYPE_GRAY``
        '''
        return TYPE_GRAY

    def size(self):
        '''
        Get display size

        Returns:
            (Rows, Columns): the display size, in characters.
        '''
        return 16, 16

    def clear(self):
        '''
        Clears the screen and positions the cursor in the upper-left corner.
        '''
        zeros = [ 0x0 for dummy in range(SH1107G._TOTAL_BYTES) ]
        self.draw(zeros, SH1107G._TOTAL_BYTES)

    def draw(self, data, bytes):
        '''
        Quickly transfer/draw bulk data (specified by data) to OLED,
        transfer size specified by bytes.

        Args:
            data (list of int): the data to transfer/draw
            bytes (int)       : data size
        '''

        # all pages fill with data
        for i in range(SH1107G._PAGE_CNT):
            if i > bytes / SH1107G._PAGE_BYTES:
                return
            self._cmd(self._BASE_PAGE_START_ADDR + i)
            self._cmd(self._BASE_LOW_COLUMN_ADDR)
            self._cmd(self._BASE_HIGH_COLUMN_ADDR)
            # fill one PAGE bytes
            for k in range(0, SH1107G._PAGE_BYTES, 32):
                # I2C limit to 32 bytes each transfer
                begin = i * SH1107G._PAGE_BYTES + k
                end   = begin + 32
                self._datas(data[begin:end])

    def home(self):
        '''
        Positions the cursor in the upper-left of the OLED.
        That is, use that location in outputting subsequent text to the display.
        '''
        self.setCursor(0, 0)

    def setCursor(self, row, column):
        '''
        Position the OLED cursor; that is, set the location
        at which subsequent text written to the OLED will be displayed.

        Args:
            row   (int): the row at which to position cursor, with 0 being the first row
            column(int): the column at which to position cursor, with 0 being the first column

	Returns:
	    None
        '''
        self._cmd(self._BASE_PAGE_START_ADDR + row)
        self._cmd(0x08 if column % 2 else self._BASE_LOW_COLUMN_ADDR)
        self._cmd(self._BASE_HIGH_COLUMN_ADDR + (column >> 1))

    def _putchar(self, c):
        asc = ord(c)
        if asc < 32 or asc > 127:
                asc = ord(' ')
        for i in range(8):
            fontmap = []
            fontmap.append(BasicFont[asc - 32][i])
            self._datas(fontmap)

    def write(self, msg):
        '''
        Write character(s) to the OLED.

        Args:
            msg (string): the character(s) to write to the display

        Returns:
            None
        '''
        for i in range(len(msg)):
            self._putchar(msg[i])

    def _backlight_on(self, en):
        self._cmd(self._DISPLAY_CMD_ON if en else self._DISPLAY_CMD_OFF)


def main():
    import time

    oled = SH1107G()
    rows, cols = oled.size()
    print("OLED model: {}".format(oled.name))
    print("OLED type : {} x {}".format(cols, rows))

    oled.backlight(False)
    time.sleep(1)

    oled.backlight(True)
    oled.setCursor(0, 0)
    oled.write("hello world!")
    oled.setCursor(0, cols - 1)
    oled.write('X')
    oled.setCursor(rows - 1, 0)
    for i in range(cols):
        oled.write(chr(ord('A') + i))

    time.sleep(3)
    oled.clear()

if __name__ == '__main__':
    main()




#!/usr/bin/env python
#
# This code is for Grove - 3-Axis Digital Accelerometer(+/-400g)
# (https://www.seeedstudio.com/Grove-3-Axis-Digital-Accelerometer-400-p-1897.html)
#
# which is a low power high performance 3-axis linear accelerometer belonging
# to the "nano" family, with digital I2C serial interface standard output.
#
# This is the library for Grove Base Hat which used to connect grove sensors for raspberry pi.
#

'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''

from __future__ import print_function
from grove.i2c import Bus
import time

# I2C address of the device
H3LIS331DL_DEFAULT_ADDRESS			= 0x19

# H3LIS331DL Register Map
H3LIS331DL_REG_WHOAMI					= 0x0F # Who Am I Register
H3LIS331DL_REG_CTRL1					= 0x20 # Control Register-1
H3LIS331DL_REG_CTRL2					= 0x21 # Control Register-2
H3LIS331DL_REG_CTRL3					= 0x22 # Control Register-3
H3LIS331DL_REG_CTRL4					= 0x23 # Control Register-4
H3LIS331DL_REG_CTRL5					= 0x24 # Control Register-5
H3LIS331DL_REG_REFERENCE				= 0x26 # Reference
H3LIS331DL_REG_STATUS					= 0x27 # Status Register
H3LIS331DL_REG_OUT_X_L					= 0x28 # X-Axis LSB
H3LIS331DL_REG_OUT_X_H					= 0x29 # X-Axis MSB
H3LIS331DL_REG_OUT_Y_L					= 0x2A # Y-Axis LSB
H3LIS331DL_REG_OUT_Y_H					= 0x2B # Y-Axis MSB
H3LIS331DL_REG_OUT_Z_L					= 0x2C # Z-Axis LSB
H3LIS331DL_REG_OUT_Z_H					= 0x2D # Z-Axis MSB

# Accl Datarate configuration
H3LIS331DL_ACCL_PM_PD					= 0x00 # Power down Mode
H3LIS331DL_ACCL_PM_NRMl					= 0x20 # Normal Mode
H3LIS331DL_ACCL_PM_0_5					= 0x40 # Low-Power Mode, ODR = 0.5Hz
H3LIS331DL_ACCL_PM_1					= 0x60 # Low-Power Mode, ODR = 1Hz
H3LIS331DL_ACCL_PM_2					= 0x80 # Low-Power Mode, ODR = 2Hz
H3LIS331DL_ACCL_PM_5					= 0xA0 # Low-Power Mode, ODR = 5Hz
H3LIS331DL_ACCL_PM_10					= 0xC0 # Low-Power Mode, ODR = 10Hz
H3LIS331DL_ACCL_DR_50					= 0x00 # ODR = 50Hz
H3LIS331DL_ACCL_DR_100					= 0x08 # ODR = 100Hz
H3LIS331DL_ACCL_DR_400					= 0x10 # ODR = 400Hz
H3LIS331DL_ACCL_DR_1000					= 0x18 # ODR = 1000Hz

# Accl Data update & Axis configuration
H3LIS331DL_ACCL_LPEN					= 0x00 # Normal Mode, Axis disabled
H3LIS331DL_ACCL_XAXIS					= 0x04 # X-Axis enabled
H3LIS331DL_ACCL_YAXIS					= 0x02 # Y-Axis enabled
H3LIS331DL_ACCL_ZAXIS					= 0x01 # Z-Axis enabled

# Acceleration Full-scale selection
H3LIS331DL_ACCL_BDU_CONT				= 0x00 # Continuous update, Normal Mode, 4-Wire Interface, LSB first
H3LIS331DL_ACCL_BDU_NOT_CONT			= 0x80 # Output registers not updated until MSB and LSB reading
H3LIS331DL_ACCL_BLE_MSB					= 0x40 # MSB first
H3LIS331DL_ACCL_RANGE_400G				= 0x30 # Full scale = +/-400g
H3LIS331DL_ACCL_RANGE_200G				= 0x10 # Full scale = +/-200g
H3LIS331DL_ACCL_RANGE_100G				= 0x00 # Full scale = +/-100g
H3LIS331DL_ACCL_SIM_3					= 0x01 # 3-Wire Interface
H3LIS331DL_RAW_DATA_MAX					= 65536

H3LIS331DL_DEFAULT_RANGE = H3LIS331DL_ACCL_RANGE_100G
H3LIS331DL_SCALE_FS = H3LIS331DL_RAW_DATA_MAX / 4 / ((H3LIS331DL_DEFAULT_RANGE >> 4) + 1)

class H3LIS331DL(object):
	def __init__ (self, address=H3LIS331DL_DEFAULT_ADDRESS):
		self._addr = address
		self._bus  = Bus()
		self.select_datarate()
		self.select_data_config()
	
	def select_datarate(self):
		"""Select the data rate of the accelerometer from the given provided values"""
		DATARATE_CONFIG = (H3LIS331DL_ACCL_PM_NRMl | H3LIS331DL_ACCL_DR_50 | H3LIS331DL_ACCL_XAXIS | H3LIS331DL_ACCL_YAXIS | H3LIS331DL_ACCL_ZAXIS)
		self._bus.write_byte_data(self._addr, H3LIS331DL_REG_CTRL1, DATARATE_CONFIG)
	
	def select_data_config(self):
		"""Select the data configuration of the accelerometer from the given provided values"""
		DATA_CONFIG = (H3LIS331DL_DEFAULT_RANGE | H3LIS331DL_ACCL_BDU_CONT)
		self._bus.write_byte_data(self._addr, H3LIS331DL_REG_CTRL4, DATA_CONFIG)
	
	def read_accl(self):
		"""Read data back from H3LIS331DL_REG_OUT_X_L(0x28), 2 bytes
		X-Axis Accl LSB, X-Axis Accl MSB"""
		data0 = self._bus.read_byte_data(self._addr, H3LIS331DL_REG_OUT_X_L)
		data1 = self._bus.read_byte_data(self._addr, H3LIS331DL_REG_OUT_X_H)
		
		xAccl = data1 * 256 + data0
		if xAccl > H3LIS331DL_RAW_DATA_MAX / 2:
			xAccl -= H3LIS331DL_RAW_DATA_MAX
		
		"""Read data back from H3LIS331DL_REG_OUT_Y_L(0x2A), 2 bytes
		Y-Axis Accl LSB, Y-Axis Accl MSB"""
		data0 = self._bus.read_byte_data(self._addr, H3LIS331DL_REG_OUT_Y_L)
		data1 = self._bus.read_byte_data(self._addr, H3LIS331DL_REG_OUT_Y_H)
		
		yAccl = data1 * 256 + data0
		if yAccl > H3LIS331DL_RAW_DATA_MAX / 2 :
			yAccl -= H3LIS331DL_RAW_DATA_MAX
		
		"""Read data back from H3LIS331DL_REG_OUT_Z_L(0x2C), 2 bytes
		Z-Axis Accl LSB, Z-Axis Accl MSB"""
		data0 = self._bus.read_byte_data(self._addr, H3LIS331DL_REG_OUT_Z_L)
		data1 = self._bus.read_byte_data(self._addr, H3LIS331DL_REG_OUT_Z_H)
		
		zAccl = data1 * 256 + data0
		if zAccl > H3LIS331DL_RAW_DATA_MAX / 2 :
			zAccl -= H3LIS331DL_RAW_DATA_MAX
		
		return {'x' : xAccl, 'y' : yAccl, 'z' : zAccl}

def main():
	h3lis331dl = H3LIS331DL()
	while True:
		h3lis331dl.select_datarate()
		h3lis331dl.select_data_config()
		time.sleep(0.2)
		accl = h3lis331dl.read_accl()
		print("Raw:    X = {0:6}   Y = {1:6}   Z = {2:6}"
			.format(accl['x'], accl['y'], accl['z']))
		print("Accel: AX = {0:6.3}g AY = {1:6.3}g AZ = {2:6.3}g"
			.format(accl['x'] / H3LIS331DL_SCALE_FS, accl['y'] / H3LIS331DL_SCALE_FS, accl['z'] / H3LIS331DL_SCALE_FS))
		time.sleep(.5)

if __name__ == '__main__':
    main()


#!/usr/bin/env python
#
# This library is for Grove - 4 Digit Display(https://wiki.seeedstudio.com/Grove-0.54_inch_Red_Quad_Alphanumeric_Display/)
# which has 4 red seven-segment displays.
# This is the library for Grove Base Hat which used to connect grove sensors for raspberry pi.
#

"""
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""
import time
from grove.i2c import Bus

display_font4 = {
    'first_dot': 0x0080,                 # 'upper .'
    'second_dot': 0x2000,                 # 'lower .'
    '0': 0x4478,
    '1': 0x0060,
    '2': 0x0758,
    '3': 0x0770,
    '4': 0x4360,
    '5': 0x4730,
    '6': 0x4738,
    '7': 0x0070,
    '8': 0x4778,
    '9': 0x4770,
    'A': 0x4378,
    'B': 0x2d70,
    'C': 0x4418,
    'D': 0x2c70,
    'E': 0x4718,
    'F': 0x4318,
    'G': 0x4538,
    'H': 0x4368,
    'I': 0x2c10,
    'J': 0x0478,
    'K': 0x2806,
    'L': 0x4408,
    'M': 0x40ea,
    'N': 0x40ec,
    'O': 0x4478,
    'P': 0x4358,
    'Q': 0x447c,
    'R': 0x435c,
    'S': 0x0494,
    'T': 0x2810,
    'U': 0x4468,
    'V': 0x500a,
    'W': 0x506c,
    'X': 0x1086,
    'Y': 0x0882,
    'Z': 0x1412,
}

display_font2 = {
    'dot': 0x4000,
    '0': 0xa145,
    '1': 0x8001,
    '2': 0x3107,
    '3': 0xb007,
    '4': 0x9043,
    '5': 0xb046,
    '6': 0xb146,
    '7': 0x8005,
    '8': 0xb147,
    '9': 0xb047,
    'A': 0x9147,
    'B': 0xb415,
    'C': 0x2144,
    'D': 0xa415,
    'E': 0x3146,
    'F': 0x1146,
    'G': 0xb144,
    'H': 0x9143,
    'I': 0x2414,
    'J': 0xa101,
    'K': 0x0c18,
    'L': 0x2140,
    'M': 0x8169,
    'N': 0x8961,
    'O': 0xa145,
    'P': 0x1147,
    'Q': 0xa945,
    'R': 0x1947,
    'S': 0x2824,
    'T': 0x0414,
    'U': 0xa141,
    'V': 0x8821,
    'W': 0x8b41,
    'X': 0x0a28,
    'Y': 0x0428,
    'Z': 0x220c,
}

REG_INIT = 0x21
DISP_ON = 0x81
REG_BRIGHT = 0xE0
BRIGHT_DARKEST = 0
BRIGHT_DEFAULT = 6
BRIGHT_HIGHEST = 15

BLINK_OFF = 0
BLINK_2HZ = 1
BLINK_1HZ = 2

FOUR_TUBES = 0
TWO_TUBES = 1


class GroveAlphanumDisplay(object):

    def __init__(self, address=0x71, brightness=BRIGHT_DEFAULT, display_type=FOUR_TUBES):
        """
        Constructor

        Args:
            address: I2C address, default is 0x71
            brightness: Startup brightness, value between 0 and 15
            display_type: Display type, can be one of 0: FOUR_TUBES and 1: TWO_TUBES
        """
        self.address = address
        self.display_type = display_type
        self.font = display_font4 if display_type == FOUR_TUBES else display_font2
        self.first_dot = False
        self.second_dot = False

        self.bus = Bus()
        self.data = [0] * 4 if self.display_type == FOUR_TUBES else 2

        self.bus.write_i2c_block_data(self.address, REG_INIT, [])
        self.bus.write_i2c_block_data(self.address, DISP_ON, [])
        self.set_brightness(brightness)

    def clear(self):
        """
        Clear display
        """
        self.data = [0] * 4 if self.display_type == FOUR_TUBES else 2
        self.first_dot = False
        self.second_dot = False
        self._show()

    def show(self, data):
        """
        Show a string on the display

        Args:
            data: String to show. If it is longer than the display size (2 or 4),
                  the string is trimmed to display length.
        """
        if type(data) is str:
            self.data = [0] * 4 if self.display_type == FOUR_TUBES else 2
            length = min(len(data), len(self.data))
            for i in range(length):
                self.data[i] = self.font.get(data[i], 0)
        else:
            raise ValueError('Not support {}'.format(type(data)))
        self._show()

    def _show(self):
        """
        Internal function to show the display data.
        First, create the I2C data to write to the display controller and then send it.
        """
        wire_bytes = [0, 0]
        byte_10 = 0
        byte_11 = 0

        if self.display_type == FOUR_TUBES:
            for d in self.data:
                wire_bytes += [d & 0xFF, (d >> 8) & 0xFF]
            for i, d in enumerate(self.data):
                if i == 0:
                    byte_10 |= (1 if (d & 0x02) else 0) << 4
                    byte_10 |= (1 if (d & 0x04) else 0) << 3
                elif i == 1:
                    byte_10 |= (1 if (d & 0x02) else 0) << 6
                    byte_11 |= (1 if (d & 0x04) else 0) << 6
                elif i == 2:
                    byte_10 |= (1 if (d & 0x02) else 0) << 5
                    byte_11 |= (1 if (d & 0x04) else 0) << 1
                else:
                    byte_11 |= (1 if (d & 0x02) else 0) << 2
                    byte_11 |= (1 if (d & 0x04) else 0) << 0

            if self.first_dot:
                byte_10 |= self.font['first_dot'] & 0xFF
                byte_11 |= (self.font['first_dot'] >> 8) & 0xFF
            if self.second_dot:
                byte_10 |= self.font['second_dot'] & 0xFF
                byte_11 |= (self.font['second_dot'] >> 8) & 0xFF

        else:
            for i in [1, 0]:
                value = self.data[i]
                if i == 1 and self.first_dot:
                    value |= self.font['dot']
                if i == 0 and self.second_dot:
                    value |= self.font['dot']
                wire_bytes += [(value >> 8) & 0xFF, value & 0xFF]

            wire_bytes += [0] * 4

        wire_bytes += [byte_10, byte_11]

        self.bus.write_i2c_block_data(self.address, 0, wire_bytes)

    def set_brightness(self, brightness):
        """
        Sets the LED brightness.

        Args:
            brightness: Brightness as integer, value between 0 and 15
        """
        if brightness > BRIGHT_HIGHEST or brightness < 0:
            brightness = BRIGHT_HIGHEST

        self.bus.write_byte(self.address, REG_BRIGHT | brightness)

    def set_blink_type(self, blink_type):
        """
        Configures the blinking of the display, can be one of:
            - 0: No blinking
            - 1: Blink with 2 Hz
            - 2: Blink with 1 Hz

        Args:
            blink_type: Blinking type
        """
        if 0 <= blink_type <= 2:
            self.bus.write_byte(self.address, 0x81 | (blink_type << 1))

    def set_dots(self, first, second):
        """
        Sets the dots in the display.

        Args:
            first: If set, the first/upper dot is on
            second: If set, the second/lower dot is on
        """
        self.first_dot = first
        self.second_dot = second
        self._show()


Grove = GroveAlphanumDisplay


def main():
    display = GroveAlphanumDisplay()

    count = 0
    brightness = 0
    while True:
        t = time.strftime("%H%M", time.localtime(time.time()))
        display.first_dot = not display.first_dot
        display.second_dot = not display.second_dot

        display.show(t)

        time.sleep(1)

        if count % 5 == 0:
            display.set_brightness(brightness % 16)
            brightness += 1

        if count % 60 == 0:
            display.set_blink_type(BLINK_OFF)
        if count % 60 == 20:
            display.set_blink_type(BLINK_1HZ)
        if count % 60 == 40:
            display.set_blink_type(BLINK_2HZ)

        count += 1


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Library for Grove - CO2 & Temperature & Humidity Sensor for Arduino (SCD30) - 3-in-1
# (https://www.seeedstudio.com/Grove-CO2-Temperature-Humidity-Sensor-SCD30-p-2911.html)
#

'''
## License

The MIT License (MIT)

Copyright (C) 2022  matsujirushi

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''

from typing import NoReturn
from grove.i2c import Bus
import struct
import time


class GroveCo2Scd30(object):
    __COMMAND_TRIGGER_CONTINUOUS_MEASUREMENT = 0x0010
    __COMMAND_STOP_CONTINUOUS_MEASUREMENT = 0x0104
    __COMMAND_SET_MEASUREMENT_INTERVAL = 0x4600
    __COMMAND_GET_DATA_READY_STATUS = 0x0202
    __COMMAND_READ_MEASUREMENT = 0x0300
    __COMMAND_ACTIVATE_ASC = 0x5306
    __COMMAND_SET_FRC = 0x5204
    __COMMAND_SET_TEMPRATURE_OFFSET = 0x5403
    __COMMAND_ALTITUDE_COMPENSATION = 0x5102
    __COMMAND_READ_FIRMWARE_VERSION = 0xd100
    __COMMAND_SOFT_RESET = 0xd304

    def __init__(self, address=0x61, bus=None):
        self.address = address
        self.bus = Bus(bus)

        self.set_measurement_interval(2)
        self.trigger_continuous_measurement()

    @staticmethod
    def _calc_crc(data: list) -> int:
        crc = 0xff

        for d in data:
            crc ^= d

            for _ in range(8):
                if crc & 0x80:
                    crc = ((crc << 1) ^ 0x31) & 0xff
                else:
                    crc = (crc << 1) & 0xff

        return crc

    def _write(self, cmd: int, data: list):
        write_data = list(struct.pack(">H", cmd))
        if data is not None:
            for d in data:
                write_data.extend(struct.pack(">H", d))
                write_data.append(GroveCo2Scd30._calc_crc(struct.pack(">H", d)))

        write_msg = self.bus.msg.write(self.address, write_data)
        self.bus.i2c_rdwr(write_msg)

    def _read(self, address: int, data_number: int) -> list:
        write_data = list(struct.pack(">H", address))

        write_msg = self.bus.msg.write(self.address, write_data)
        self.bus.i2c_rdwr(write_msg)

        read_msg = self.bus.msg.read(self.address, 3 * data_number)
        self.bus.i2c_rdwr(read_msg)

        result = []
        for i in range(data_number):
            d = read_msg.buf[i*3:i*3+2]
            if GroveCo2Scd30._calc_crc(d) != read_msg.buf[i*3+2][0]:
                raise ValueError("CRC mismatch")

            result.append(struct.unpack(">H", d)[0])

        return result

    def trigger_continuous_measurement(self, pressure: int = 0):
        self._write(self.__COMMAND_TRIGGER_CONTINUOUS_MEASUREMENT, [pressure])

    def stop_continuous_measurement(self):
        self._write(self.__COMMAND_STOP_CONTINUOUS_MEASUREMENT, None)

    def set_measurement_interval(self, interval: int):
        self._write(self.__COMMAND_SET_MEASUREMENT_INTERVAL, [interval])

    def get_measurement_interval(self) -> int:
        data = self._read(self.__COMMAND_SET_MEASUREMENT_INTERVAL, 1)

        return data[0]

    def get_data_ready_status(self) -> bool:
        data = self._read(self.__COMMAND_GET_DATA_READY_STATUS, 1)

        return True if data[0] == 1 else False

    def read_measurement(self) -> tuple:
        data = self._read(self.__COMMAND_READ_MEASUREMENT, 6)

        data_bytes = struct.pack(">HHHHHH", data[0], data[1], data[2], data[3], data[4], data[5])
        data_floats = struct.unpack(">fff", data_bytes)
        co2 = data_floats[0]
        temp = data_floats[1]
        humi = data_floats[2]

        return co2, temp, humi

    def set_forced_recalibration(self, co2: float):
        self._write(self.__COMMAND_SET_FRC, [int(co2)])

    def set_automatic_self_calibration(self, activate: bool):
        self._write(self.__COMMAND_ACTIVATE_ASC, [1 if activate else 0])

    def get_automatic_self_calibration(self) -> bool:
        data = self._read(self.__COMMAND_ACTIVATE_ASC, 1)

        return True if data[0] == 1 else False

    def set_temperature_offset(self, offset: float):
        self._write(self.__COMMAND_SET_TEMPRATURE_OFFSET, [int(offset * 100)])

    def get_temperature_offset(self) -> float:
        data = self._read(self.__COMMAND_SET_TEMPRATURE_OFFSET, 1)

        return float(data[0]) / 100

    def set_altitude_compensation(self, altitude: int):
        self._write(self.__COMMAND_ALTITUDE_COMPENSATION, [altitude])

    def get_altitude_compensation(self) -> int:
        data = self._read(self.__COMMAND_ALTITUDE_COMPENSATION, 1)

        return data[0]

    def read(self) -> tuple:
        if not self.get_data_ready_status():
            return None

        return self.read_measurement()


def main() -> NoReturn:
    sensor = GroveCo2Scd30()

    while True:
        if sensor.get_data_ready_status():
            co2, temperature, humidity = sensor.read()
            print(f"CO2 concentration is {co2:.1f} ppm")
            print(f"Temperature in Celsius is {temperature:.2f} C")
            print(f"Relative Humidity is {humidity:.2f} %")

        time.sleep(1)


if __name__ == "__main__":
    main()
